import json
import pytest
import requests
import requests_mock
from base64 import b64encode
from conftest import get_response
from kpm.registry import Registry
import kpm


def test_headers_without_auth():
    r = Registry()
    assert sorted(r.headers.keys()) == ['Content-Type', 'User-Agent']
    assert r.headers["Content-Type"] == "application/json"
    assert r.headers["User-Agent"] == "kpmpy-cli: %s" % kpm.__version__


def test_headers_with_auth():
    r = Registry()
    r.auth.token = "titi"
    assert sorted(r.headers.keys()) == ["Authorization", 'Content-Type', 'User-Agent']
    assert r.headers["Authorization"] == "titi"
    assert r.headers["Content-Type"] == "application/json"
    assert r.headers["User-Agent"] == "kpmpy-cli: %s" % kpm.__version__


def test_default_endpoint():
    r = Registry(endpoint=None)
    assert r.endpoint.geturl() == "https://api.kpm.sh"


def test_url():
    r = Registry(endpoint="http://test.com")
    assert r._url("/test") == "http://test.com/api/v1/test"


def test_pull():
    r = Registry()
    with requests_mock.mock() as m:
        response = 'package_data'
        m.get("https://api.kpm.sh/api/v1/packages/orga/p1/pull", text=response)
        assert r.pull("orga/p1") == response


def test_pull_with_version():
    r = Registry()
    with requests_mock.mock() as m:
        response = 'package_data'
        m.get("https://api.kpm.sh/api/v1/packages/orga/p1/pull?version=1.0.1", complete_qs=True, text=response)
        assert r.pull("orga/p1", version="1.0.1") == response


def test_list_packages():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages", text=response)
        assert json.dumps(r.list_packages()) == response


def test_list_packages_username():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages?username=ant31", complete_qs=True, text=response)
        assert json.dumps(r.list_packages(user="ant31")) == response


def test_list_packages_orga():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages?organization=ant31", complete_qs=True, text=response)
        assert json.dumps(r.list_packages(organization="ant31")) == response


def test_list_packages_orga_and_user():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages?username=titi&organization=ant31", complete_qs=True, text=response)
        assert json.dumps(r.list_packages(user="titi", organization="ant31")) == response


def test_generate():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages/ant31/kube-ui/generate", complete_qs=True, text=response)
        assert json.dumps(r.generate(name="ant31/kube-ui")) == response


def test_generate_with_params():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.get("https://api.kpm.sh/api/v1/packages/ant31/kube-ui/generate?tarball=true&variables=a&variables=c&version=1.3.4&namespace=testns", complete_qs=True, text=response)
        assert json.dumps(r.generate(name="ant31/kube-ui", namespace="testns", variables={"a": "b", "c": "d"}, version="1.3.4", tarball=True)) == response


def test_signup():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"email": "al@kpm.sh", "token": "signup_token"}'
        m.post("https://api.kpm.sh/api/v1/users?user%5Bpassword%5D=plop&user%5Busername%5D=ant31&user%5Bpassword_confirmation%5D=plop&user%5Bemail%5D=al%40kpm.sh", complete_qs=True, text=response)
        sign_r = r.signup("ant31", "plop", "plop", "al@kpm.sh")
        assert json.dumps(sign_r) == json.dumps(json.loads(response))
        assert r.auth.token == "signup_token"


# @TODO
def test_signup_existing():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"email": "al@kpm.sh", "token": "signup_token"}'
        m.post("https://api.kpm.sh/api/v1/users?user%5Bpassword%5D=plop&user%5Busername%5D=ant31&user%5Bpassword_confirmation%5D=plop&user%5Bemail%5D=al%40kpm.sh", complete_qs=True, text=response, status_code=401)
        with pytest.raises(requests.HTTPError):
            sign_r = r.signup("ant31", "plop", "plop", "al@kpm.sh")


def test_login():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"email": "al@kpm.sh", "token": "login_token"}'
        m.post("https://api.kpm.sh/api/v1/users/login?user%5Bpassword%5D=plop&user%5Busername%5D=ant31", complete_qs=True, text=response)
        login_r = r.login("ant31", "plop")
        assert json.dumps(login_r) == json.dumps(json.loads(response))
        assert r.auth.token == "login_token"


def test_login_failed():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"email": "al@kpm.sh", "token": "login_token"}'
        m.post("https://api.kpm.sh/api/v1/users/login?user%5Bpassword%5D=plop&user%5Busername%5D=ant31",
              complete_qs=True,
              text=response, status_code=401)
        with pytest.raises(requests.HTTPError):
            login_r = r.login("ant31", "plop")


def test_delete_package():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.delete("https://api.kpm.sh/api/v1/packages/ant31/kube-ui", complete_qs=True, text=response)
        assert r.delete_package(name="ant31/kube-ui") is True


def test_delete_package_version():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.delete("https://api.kpm.sh/api/v1/packages/ant31/kube-ui?version=1.4.3", complete_qs=True, text=response)
        assert r.delete_package(name="ant31/kube-ui", version="1.4.3") is True


def test_delete_package_unauthorized():
    r = Registry()
    with requests_mock.mock() as m:
        response = '{"packages": "true"}'
        m.delete("https://api.kpm.sh/api/v1/packages/ant31/kube-ui",
                 complete_qs=True,
                 text=response,
                 status_code=401)
        with pytest.raises(requests.HTTPError):
            r.delete_package(name="ant31/kube-ui")


def test_push_unauthorized(kubeui_blob):
    r = Registry()
    with requests_mock.mock() as m:
        body = {"blob": b64encode(kubeui_blob)}
        response = '{"packages": "true"}'
        m.post("https://api.kpm.sh/api/v1/packages/ant31/kube-ui?force=false",
                 complete_qs=True,
                 text=response,
                 status_code=401)
        with pytest.raises(requests.HTTPError):
            r.push(name="ant31/kube-ui", body=body)


def test_push(kubeui_blob):
    body = {"blob": b64encode(kubeui_blob)}
    r = Registry()
    response = '{"packages": "true"}'
    with requests_mock.mock() as m:
        m.post("https://api.kpm.sh/api/v1/packages/ant31/kube-ui?force=false",
               complete_qs=True,
               text=response)
        assert json.dumps(r.push(name="ant31/kube-ui", body=body)) == json.dumps(json.loads(response))



def test_push_force(kubeui_blob):
    body = {"blob": b64encode(kubeui_blob)}
    r = Registry()
    response = '{"packages": "true"}'
    with requests_mock.mock() as m:
        m.post("https://api.kpm.sh/api/v1/packages/ant31/kube-ui?force=true",
               complete_qs=True,
               text=response)
        assert json.dumps(r.push(name="ant31/kube-ui", body=body, force=True)) == json.dumps(json.loads(response))



from flask import jsonify, request, Blueprint, current_app
import kpm


info_app = Blueprint('info', __name__,)


@info_app.before_request
def pre_request_logging():
    jsonbody = request.get_json(force=True, silent=True)
    values = request.values.to_dict()
    if jsonbody:
        values.update(jsonbody)

    current_app.logger.info("request", extra={
        "remote_addr": request.remote_addr,
        "http_method": request.method,
        "original_url": request.url,
        "path": request.path,
        "data":  values,
        "headers": dict(request.headers.to_list())}
    )


@info_app.route("/version")
def version():
    return jsonify({"kpm": kpm.__version__})


@info_app.route("/test_timeout")
def test_timeout():
    import time
    time.sleep(60)
    return jsonify({"kpm": kpm.__version__})



import json
from flask import jsonify, request, Blueprint, current_app
from kpm.kub import Kub
from werkzeug.exceptions import BadRequest


builder_app = Blueprint('builder', __name__,)


def _build(package):
    name = package
    args = {}
    version = None
    try:
        body = request.get_json()
    except BadRequest:
        body = {}
    try:
        args = request.args
    except BadRequest:
        args = {}
    namespace = args.get('namespace', default='default')
    variables = {'namespace': namespace}
    if body is None:
        body = {}
    body.update(args.to_dict())
    if 'variables' in body:
        variables = body['variables']
    if 'namespace' in body:
        namespace = body['namespace']
        variables['namespace'] = namespace
    if 'version' in body:
        version = body['version']
    k = Kub(name, endpoint=current_app.config['KPM_URI'], variables=variables, namespace=namespace, version=version)
    return k


@builder_app.route("/api/v1/packages/<path:package>/file/<path:filepath>")
def show_file(package, filepath):
    k = Kub(package, endpoint=current_app.config['KPM_URI'])
    return k.package.file(filepath)


@builder_app.route("/api/v1/packages/<path:package>/tree")
def tree(package):
    k = Kub(package, endpoint=current_app.config['KPM_URI'])
    return json.dumps(k.package.tree())


@builder_app.route("/api/v1/packages/<path:package>/generate", methods=['POST', 'GET'])
def build(package):
    k = _build(package)
    return jsonify(k.build())


@builder_app.route("/api/v1/packages/<path:package>/generate-tar", methods=['POST', 'GET'])
def build_tar(package):
    k = _build(package)
    resp = current_app.make_response(k.build_tar())
    resp.mimetype = 'application/tar'
    resp.headers['Content-Disposition'] = 'filename="%s_%s.tar.gz"' % (k.name.replace("/", "_"), k.version)
    return resp



import pytest
import kpm.api.registry as api
import kpm
import etcd
from kpm.api.exception import (
    InvalidVersion,
    PackageAlreadyExists,
    )

# from kpm.api.exception import (ApiException,
#                                        InvalidUsage,
#                                        InvalidVersion,
#                                        PackageAlreadyExists,
#                                        PackageNotFound,
#                                        PackageVersionNotFound)


class MockEtcdResult(object):
    def __init__(self, result):
        self.result = result

    @property
    def key(self):
        return self.result

    @property
    def value(self):
        return "value"


class MockEtcdResults(object):
    def __init__(self, results):
        self.results = results

    @property
    def children(self):
        return [MockEtcdResult(x) for x in self.results]


def test_showversion(client):
    import kpm
    res = client.get("/version")
    assert res.json == {"kpm": kpm.__version__}


def test_pathfor():
    assert api.pathfor("a/b", "1.4.3") == "kpm/packages/a/b/1.4.3"


def test_check_data_validversion():
    assert api.check_data("a/b", "1.4.5-alpha", "fdsf") is None


def test_check_data_invalidversion():
    with pytest.raises(InvalidVersion):
        assert api.check_data("a/b", "1.4.5a-alpha", "fdsf")


@pytest.fixture()
def getversions(monkeypatch):
    def read(path, recursive=True):
        assert path == "kpm/packages/ant31/rocketchat"
        return MockEtcdResults(["kpm/packages/ant31/rocketchat/1.3.0",
                                "kpm/packages/ant31/rocketchat/1.3.2-rc2",
                                "kpm/packages/ant31/rocketchat/1.8.2-rc2",
                                "kpm/packages/ant31/rocketchat/1.4.2",
                                "kpm/packages/ant31/rocketchat/1.0.0",
                                "kpm/packages/ant31/rocketchat/1.2.0"])
    monkeypatch.setattr("kpm.api.registry.etcd_client.read", read)


def test_getversions(getversions):
    assert api.getversions("ant31/rocketchat") == ['1.3.0', '1.3.2-rc2', '1.8.2-rc2', '1.4.2', '1.0.0', '1.2.0']


def test_getversions_empty(monkeypatch):
    def read(path, recursive=True):
        assert path == "kpm/packages/ant31/rocketchat"
        return MockEtcdResults([])
    monkeypatch.setattr("kpm.api.registry.etcd_client.read", read)
    assert api.getversions("ant31/rocketchat") == []


def test_getversion_latest(getversions):
    assert str(api.getversion("ant31/rocketchat", "latest")) == "1.8.2-rc2"


def test_getversion_stable_none(getversions):
    assert str(api.getversion("ant31/rocketchat", None, True)) == "1.4.2"


def test_getversion_invalid(getversions):
    with pytest.raises(InvalidVersion):
        str(api.getversion("ant31/rocketchat", "==4.25a"))


def test_getversion_prerelease(getversions):
    str(api.getversion("ant31/rocketchat", ">=0-")) == "1.8.2-rc2"


def test_push_etcd(monkeypatch):
    def write(path, data, prevExist):
        assert path == "kpm/packages/a/b/4"
        assert data == "value"
        return True
    monkeypatch.setattr("kpm.api.registry.etcd_client.write", write)
    api.push_etcd("a/b", 4, "value")


def test_push_etcd_exist(monkeypatch):
    def write(path, data, prevExist):
        assert path == "kpm/packages/a/b/4"
        assert data == "value"
        raise etcd.EtcdAlreadyExist
    monkeypatch.setattr("kpm.api.registry.etcd_client.write", write)
    with pytest.raises(PackageAlreadyExists):
        api.push_etcd("a/b", 4, "value")



import yaml
import json
from base64 import b64decode
from flask import jsonify, request, Blueprint, current_app
import semantic_version
from kpm.packager import Package
import kpm.semver as semver
from kpm.semver import last_version, select_version
from kpm.api.exception import (ApiException,
                               InvalidUsage,
                               InvalidVersion,
                               PackageAlreadyExists,
                               PackageNotFound,
                               PackageVersionNotFound)

import etcd

registry_app = Blueprint('registry', __name__,)

etcd_client = etcd.Client(port=2379)

ETCD_PREFIX = "kpm/packages/"


def pathfor(package, version):
    return ETCD_PREFIX + "%s/%s" % (package, version)


def getvalues():
    jsonbody = request.get_json(force=True, silent=True)
    values = request.values.to_dict()
    if jsonbody:
        values.update(jsonbody)
    return values


def check_data(package, version, blob):
    try:
        semantic_version.Version(version)
    except ValueError as e:
        raise InvalidVersion(e.message, {"version": version})
    return None


def getversions(package):
    path = ETCD_PREFIX + package
    r = etcd_client.read(path, recursive=True)
    versions = []
    for p in r.children:
        version = p.key.split("/")[-1]
        versions.append(version)
    return versions


def getversion(package, version, stable=False):
    versions = getversions(package)
    if version is None or version == 'latest':
        return last_version(versions, stable)
    else:
        try:
            return select_version(versions, str(version), stable)
        except ValueError as e:
            raise InvalidVersion(e.message, {"version": version})


def push_etcd(package, version, data, force=False):
    path = pathfor(package, version)
    try:
        etcd_client.write(path, data, prevExist=force)
    except etcd.EtcdAlreadyExist as e:
        raise PackageAlreadyExists(e.message, {"package": path})


@registry_app.errorhandler(etcd.EtcdKeyNotFound)
def render_etcdkeyerror(error):
    package = error.payload['cause']
    return render_error(PackageNotFound("Package not found: %s" % package, {"package": package}))


@registry_app.errorhandler(PackageAlreadyExists)
@registry_app.errorhandler(PackageNotFound)
@registry_app.errorhandler(PackageVersionNotFound)
@registry_app.errorhandler(ApiException)
@registry_app.errorhandler(InvalidUsage)
def render_error(error):
    response = jsonify({"error": error.to_dict()})
    response.status_code = error.status_code
    return response


@registry_app.route("/test_error")
def test_error():
    raise InvalidUsage("error message", {"path": request.path})


def get_package(package, values):
    # if version is None; Find latest version

    version_query = values.get("version", 'latest')
    version = getversion(package, version_query)
    if version is None:
        raise PackageVersionNotFound("No version match '%s' for package '%s'" % (version_query, package),
                                     {"package": package, "version_query": version_query})
    path = pathfor(package, str(version))
    package_data = etcd_client.read(path)
    return package_data


@registry_app.route("/api/v1/packages/<organization>/<name>/pull", methods=['GET'], strict_slashes=False)
def pull(organization, name):
    current_app.logger.info("pull %s,%s", organization, name)
    values = getvalues()
    package = "%s/%s" % (organization, name)
    package_data = get_package(package, values)
    if 'format' in values and values['format'] == 'json':
        resp = jsonify({"package": package, "kub": package_data.value})
    else:
        resp = current_app.make_response(b64decode(package_data.value))
        resp.headers['Content-Disposition'] = 'filename="%s_%s.tar.gz"' % (package.replace("/", "_"),
                                                                           package_data.key.split("/")[-1])
        resp.mimetype = 'application/x-gzip'
    return resp


@registry_app.route("/api/v1/packages/<organization>/<name>", methods=['POST'], strict_slashes=False)
@registry_app.route("/api/v1/packages", methods=['POST'], strict_slashes=False)
def push(organization=None, name=None):

    values = getvalues()
    blob = values['blob']
    package = values['package']
    print values
    version = values['version']
    force = False
    if 'force' in values:
        force = 'true' == values['force']
    check_data(package, version, blob)
    push_etcd(package, version, blob, force=force)
    return jsonify({"status": "ok"})


@registry_app.route("/api/v1/packages", methods=['GET'], strict_slashes=False)
def list_packages():
    values = getvalues()
    path = ETCD_PREFIX
    if 'organization' in values:
        path += "/%s" % values['organization']

    packages = etcd_client.read(path, recursive=True)
    r = {}

    for child in packages.children:
        p = child.key.split(ETCD_PREFIX)[1]
        organization, name, version = p.split("/")
        package = "%s/%s" % (organization, name)
        if package not in r:
            r[package] = {"name": package, 'available_versions': [], 'version': None}
        r[package]['available_versions'].append(version)

    for _, v in r.iteritems():
        v['available_versions'] = [str(x) for x in sorted(semver.versions(v['available_versions'], False),
                                                          reverse=True)]
        v['version'] = v['available_versions'][0]

    resp = current_app.make_response(json.dumps(r.values()))
    resp.mimetype = 'application/json'
    return resp


@registry_app.route("/api/v1/packages/<organization>/<name>", methods=['GET'], strict_slashes=False)
def show_package(organization, name):
    package = "%s/%s" % (organization, name)
    values = getvalues()
    package_data = get_package(package, values)
    p = Package(b64decode(package_data.value))
    manifest = yaml.load(p.manifest)
    stable = False
    if 'stable' in values and values['stable'] == 'true':
        stable = True

    response = {"manifest": manifest,
                "version": manifest['package']['version'],
                "name":  package,
                "available_versions": [str(x) for x in sorted(semver.versions(getversions(package), stable),
                                                              reverse=True)]}
    if 'pull' in values and values['pull'] == 'true':
        response['kub'] = package_data.value
    return jsonify(response)


@registry_app.route("/api/v1/packages/<organization>/<name>", methods=['DELETE'], strict_slashes=False)
def delete_package(organization, name):
    package = "%s/%s" % (organization, name)
    values = getvalues()
    package_data = get_package(package, values)
    etcd_client.delete(package_data.key)
    return jsonify({"status": "delete", "key": package_data.key})



#!/usr/bin/env python
import argparse
import getpass

import os
from kpm.auth import KpmAuth
import kpm.registry as registry
from kpm.registry import Registry
from kpm.packager import pack_kub, Package
import kpm.manifest
import kpm.deploy
from kpm.display import print_packages
from kpm.new import new_package
from kpm.console import KubernetesExec
import base64


def new(options):
    new_package(options.package[0], options.directory, options.with_comments)


def show(options):
    r = Registry(options.registry_host)
    result = r.pull(options.package[0], version=options.version)
    p = Package(result)
    if options.tree:
        print "\n".join(p.tree())
    elif options.file:
        print p.file(options.file)
    else:
        print p.manifest


def install(options):
    kpm.deploy.deploy(options.package[0],
                      version=options.version,
                      dest=options.tmpdir,
                      namespace=options.namespace,
                      force=options.force,
                      dry=options.dry_run,
                      endpoint=options.registry_host,
                      proxy=options.api_proxy)


def remove(options):
    kpm.deploy.delete(options.package[0],
                      version=options.version,
                      dest=options.tmpdir,
                      namespace=options.namespace,
                      dry=options.dry_run,
                      endpoint=options.registry_host,
                      proxy=options.api_proxy)


def pull(options):
    r = Registry(options.registry_host)
    result = r.pull(options.package[0], version=options.version)
    p = Package(result)
    path = os.path.join(options.directory, kpm.manifest.Manifest(p).package_name())
    p.extract(path)


def exec_cmd(options):
    c = KubernetesExec(options.name,
                       cmd=" ".join(options.cmd),
                       namespace=options.namespace,
                       container=options.container,
                       kind=options.kind)
    c.call()


def push(options):
    r = Registry(options.registry_host)
    # @TODO: Override organization
    manifest = kpm.manifest.Manifest(path=".")
    # @TODO: Pack in memory
    kubepath = os.path.join(".", manifest.package_name() + ".tar.gz")
    pack_kub(kubepath)
    f = open(kubepath, 'rb')
    r.push(manifest.package['name'], {"name": manifest.package['name'],
                                      "version": manifest.package['version'],
                                      "blob": base64.b64encode(f.read())}, options.force)
    f.close()
    os.remove(kubepath)
    print "package: %s (%s) pushed" % (manifest.package['name'],
                                       manifest.package['version'])


def list_packages(options):
    r = Registry(options.registry_host)
    response = r.list_packages(user=options.user, organization=options.organization)
    print_packages(response)


def login(options):
    r = Registry(options.registry_host)
    if options.user is not None:
        user = options.user
    else:
        user = raw_input("Username: ")
    if options.password is not None:
        p1 = options.password
    else:
        p1 = getpass.getpass()

    if options.signup:
        if options.password is not None:
            p2 = p1
        else:
            p2 = getpass.getpass('Password confirmation: ')
        if options.email is not None:
            email = options.email
        else:
            email = raw_input("Email: ")
        if p1 != p2:
            print "Error: password mismatch"
            exit(1)
        r.signup(user, p1, p2, email)
        print ' >>> Registration complete'
    else:
        r.login(user, p1)
        print ' >>> Login succeeded'


def logout(options):
    KpmAuth().delete_token()
    print ' >>> Logout complete'


def delete_package(options):
    r = Registry(options.registry_host)
    r.delete_package(options.package[0], version=options.version)
    print "Package %s deleted" % (options.package[0])


def get_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument("--namespace",
                        help="namespace to deploy the application")

    subparsers = parser.add_subparsers(help='command help')

    # PUSH
    push_parser = subparsers.add_parser('push', help='push a package to the registry')
    push_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                             help='registry API url')
    push_parser.add_argument("-o", "--organization", nargs="?", default=None,
                             help="push to another organization")
    push_parser.add_argument("-f", "--force", action='store_true', default=False,
                             help="force push")

    push_parser.set_defaults(func=push)

    # PULL
    pull_parser = subparsers.add_parser('pull', help='download a package and extract it')
    pull_parser.add_argument('package', nargs=1, help="package-name")
    pull_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                             help='registry API url')
    pull_parser.add_argument("--directory", nargs="?", default=".",
                             help="destionation directory")
    pull_parser.add_argument("-v", "--version", nargs="?", default=None,
                             help="package version")

    pull_parser.set_defaults(func=pull)

    # Show
    show_parser = subparsers.add_parser('show', help='print the package manifest')
    show_parser.add_argument('package', nargs=1, help="package-name")
    show_parser.add_argument('--tree', help="List files inside the package", action='store_true', default=False)
    show_parser.add_argument('-f', '--file', nargs="?", help="Display a file", default=None)
    show_parser.add_argument("-v", "--version", nargs="?", default=None,
                             help="package version")

    show_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                             help='registry API url')

    show_parser.set_defaults(func=show)

    # new
    new_parser = subparsers.add_parser('new', help='initiate a new package')
    new_parser.add_argument('package', nargs=1, help="package-name")
    new_parser.add_argument("--directory",  nargs="?", default=".",
                            help="destionation directory")
    new_parser.add_argument("--with-comments", action='store_true', default=False,
                            help="Add 'help' comments to manifest")

    new_parser.set_defaults(func=new)

    # Login
    login_parser = subparsers.add_parser('login', help='login')
    login_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                              help='registry API url')
    login_parser.add_argument("-s", "--signup", action='store_true', default=False,
                              help="Create a new account and login")
    login_parser.add_argument("-u", "--user", nargs="?", default=None,
                              help="username")
    login_parser.add_argument("-p", "--password", nargs="?", default=None,
                              help="password")
    login_parser.add_argument("-e", "--email", nargs="?", default=None,
                              help="email for signup")

    login_parser.set_defaults(func=login)

    # Logout
    logout_parser = subparsers.add_parser('logout', help='logout')
    logout_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                               help='registry API url')
    logout_parser.set_defaults(func=logout)

    # Install
    install_parser = subparsers.add_parser('deploy', help='deploy a package on kubernetes')
    install_parser.add_argument('package', nargs=1, help="package-name")
    install_parser.add_argument("--tmpdir", nargs="?", default="/tmp/",
                                help="directory used to extract resources")
    install_parser.add_argument("--dry-run", action='store_true', default=False,
                                help="do not create the resources on kubernetes")
    install_parser.add_argument("--namespace", nargs="?",
                                help="kubernetes namespace", default=None)
    install_parser.add_argument("--api-proxy", nargs="?",
                                help="kubectl proxy url", const="http://localhost:8001")
    install_parser.add_argument("-v", "--version", nargs="?",
                                help="package VERSION", default=None)

    install_parser.add_argument("--force", action='store_true', default=False,
                                help="force upgrade, delete and recreate resources")
    install_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                                help='registry API url')
    install_parser.set_defaults(func=install)

    # remove
    remove_parser = subparsers.add_parser('remove', help='remove a package from kubernetes')
    remove_parser.add_argument('package', nargs=1, help="package-name")
    remove_parser.add_argument("--tmpdir", nargs="?", default="/tmp/",
                               help="directory used to extract resources")
    remove_parser.add_argument("--dry-run", action='store_true', default=False,
                               help="Does not delete the resources on kubernetes")
    remove_parser.add_argument("--namespace", nargs="?",
                               help="kubernetes namespace", default=None)
    remove_parser.add_argument("--api-proxy", nargs="?",
                               help="kubectl proxy url", const="http://localhost:8001")

    remove_parser.add_argument("-v", "--version", nargs="?",
                               help="package VERSION to delete", default=None)

    remove_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                               help='registry API url')
    remove_parser.set_defaults(func=remove)

    # list
    list_parser = subparsers.add_parser('list', help='list packages')
    list_parser.add_argument("-u", "--user", nargs="?", default=None,
                             help="list packages owned by USER")
    list_parser.add_argument("-o", "--organization", nargs="?", default=None,
                             help="list ORGANIZATION packages")
    list_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                             help='registry API url')

    list_parser.set_defaults(func=list_packages)

    #  DELETE-PACKAGE
    delete_parser = subparsers.add_parser('delete-package', help='delete package from the registry')
    delete_parser.add_argument('package', nargs=1, help="package-name")
    delete_parser.add_argument("-H", "--registry-host", nargs="?", default=registry.DEFAULT_REGISTRY,
                               help='registry API url')
    delete_parser.add_argument("-v", "--version", nargs="?", default=None,
                               help="package version")

    delete_parser.set_defaults(func=delete_package)

    #  EXEC
    exec_parser = subparsers.add_parser('exec', help='exec a command in pod from the RC or RS name.\
    It executes the command on the first matching pod')
    exec_parser.add_argument('cmd', nargs='+', help="command to execute")
    exec_parser.add_argument("--namespace", nargs="?",
                             help="kubernetes namespace", default='default')

    exec_parser.add_argument('-k', '--kind', choices=['deployment', 'rs', 'rc'], nargs="?",
                             help="deployment, rc or rs", default='rc')
    exec_parser.add_argument('-n', '--name', help="resource name", default='rs')
    exec_parser.add_argument('-c', '--container', nargs='?', help="container name", default=None)

    exec_parser.set_defaults(func=exec_cmd)

    return parser



import pytest
import os.path

from kpm.packager import unpack_kub
import hashlib


TAR_MD5SUM = "8ccd8af6ef21af7309839f1c521b6354"
KUBEUI_FILES = ["manifest.yaml",
                "README.md",
                "templates/kube-ui-rc.yaml",
                "templates/kube-ui-svc.yaml"]


def _check_kub(path):
    for f in KUBEUI_FILES:
        assert os.path.exists(os.path.join(str(path), f))
    assert os.path.exists(os.path.join(str(path), "templates/another_file_to_ignore.cfg")) is False
    assert os.path.exists(os.path.join(str(path), "file_to_ignore.yaml")) is False


def test_pack_kub_with_authorized_only(pack_tar, tmpdir):
    import tarfile
    tar = tarfile.open(pack_tar, "r")
    tar.extractall(str(tmpdir))
    _check_kub(str(tmpdir))


def test_unpack_kub(pack_tar, tmpdir):
    unpack_kub(pack_tar, str(tmpdir))
    _check_kub(str(tmpdir))


def test_extract(kubeui_package, tmpdir):
    d = tmpdir.mkdir("extract")
    kubeui_package.extract(str(d))
    _check_kub(str(d))


def test_pack(kubeui_package, tmpdir):
    d = str(tmpdir.mkdir("pack")) + "/kube-ui.tar"
    kubeui_package.pack(d)
    assert hashlib.md5(open(d, "r").read()).hexdigest() == TAR_MD5SUM


def test_tree(kubeui_package):
    files = kubeui_package.tree()
    assert sorted(files) == sorted(KUBEUI_FILES)


def test_tree_filter(kubeui_package):
    files = kubeui_package.tree("templates")
    assert sorted(files) == sorted(["templates/kube-ui-rc.yaml", "templates/kube-ui-svc.yaml"])


def test_file(kubeui_package):
    manifest = kubeui_package.file("manifest.yaml")
    assert manifest == open("tests/data/kube-ui/manifest.yaml", "r").read()


def test_manifest(kubeui_package):
    assert kubeui_package.manifest == open("tests/data/kube-ui/manifest.yaml", "r").read()



import json
import subprocess
import pytest
import os
from kpm.api.app import create_app


@pytest.fixture
def app():
    app = create_app()
    return app


@pytest.fixture(scope='module')
def jinja_env():
    import kpm.kub
    return kpm.kub.jinja_env


@pytest.fixture()
def fake_home(monkeypatch, tmpdir):
    home = tmpdir.mkdir('home')
    monkeypatch.setenv("HOME", home)
    return home


def get_response(name, kind):
    f = open("tests/data/responses/%s-%s.json" % (name, kind))
    r = f.read()
    f.close()
    return r


@pytest.fixture(scope="module")
def kubeui_package():
    import kpm.packager
    with open("./tests/data/kube-ui.tar.gz", "rb") as f:
        package = kpm.packager.Package(f.read())
    print package.files
    return package


@pytest.fixture(scope="module")
def kubeui_blob():
    import kpm.packager
    with open("./tests/data/kube-ui.tar.gz", "rb") as f:
        package = f.read()
    return package


@pytest.fixture(scope="module")
def deploy_json():
    f = open("tests/data/kube-ui_release.json", 'r')
    r = f.read()
    f.close()
    return r


@pytest.fixture()
def package_dir(monkeypatch):
    monkeypatch.chdir("tests/data/kube-ui")


@pytest.fixture()
def pack_tar(package_dir, tmpdir):
    from kpm.packager import pack_kub
    kub = os.path.join(str(tmpdir.mkdir("tars")), "kube-ui.tar.gz")
    pack_kub(kub)
    return kub


@pytest.fixture(scope="module")
def deploy(deploy_json):
    return json.loads(deploy_json)


@pytest.fixture(scope="module")
def ns_resource(deploy):
    kubeui = deploy["deploy"][0]
    return kubeui['resources'][0]


@pytest.fixture(scope="module")
def rc_resource(deploy):
    kubeui = deploy["deploy"][0]
    return kubeui['resources'][1]


@pytest.fixture(scope="module")
def svc_resource(deploy):
    kubeui = deploy["deploy"][0]
    return kubeui['resources'][2]


@pytest.fixture()
def subcall_cmd(monkeypatch):
    def get_cmd(cmd, stderr="err"):
        return " ".join(cmd)
    monkeypatch.setattr("subprocess.check_output", get_cmd)


@pytest.fixture()
def subcall_cmd_error(monkeypatch):
    def get_cmd(cmd, stderr="err"):
        raise subprocess.CalledProcessError("a", "b", "c")
    monkeypatch.setattr("subprocess.check_output", get_cmd)



@pytest.fixture()
def subcall_get_assert(monkeypatch):
    def get_cmd(cmd, stderr="err"):
        kind, name = cmd[2], cmd[3]
        assert " ".join(cmd) == "kubectl get %s %s -o json --namespace testns" % (kind, name)
        return get_response(name, kind)
    monkeypatch.setattr("subprocess.check_output", get_cmd)


@pytest.fixture()
def subcall_all(monkeypatch):
    def get_cmd(cmd, stderr="err"):
        action, kind, name = cmd[1], cmd[2], cmd[3]
        if action != "create":
            return get_response(name, kind)
        else:
            with open(name, 'r') as f:
                return f.read()
    monkeypatch.setattr("subprocess.check_output", get_cmd)


@pytest.fixture()
def subcall_delete(monkeypatch):
    def get_cmd(cmd, stderr="err"):
        action, kind, name = cmd[1], cmd[2], cmd[3]
        if action == "get":
            assert " ".join(cmd) == "kubectl get %s %s -o json --namespace testns" % (kind, name)
        elif action == "delete":
            assert " ".join(cmd) == "kubectl delete %s %s --namespace testns" % (kind, name)
        return get_response(name, kind)
    monkeypatch.setattr("subprocess.check_output", get_cmd)



import pytest
import kpm.semver as semver
import random
import copy


@pytest.fixture(scope='module')
def version_list():
    l = ["1.4.0",
         "1.6.0-alpha",
         "1.6.0-alpha.1",
         "1.6.0-alpha.beta",
         "1.6.0-beta",
         "1.6.0-beta.2",
         "1.6.0-beta.11",
         "1.6.0-rc.1",
         "1.6.0",
         "1.7.0-rc"]
    return l


@pytest.fixture(scope='module')
def version_query():
    return ['0.1.0',
            '0.3.0',
            '0.3.2-rc',
            '0.5.0',
            '0.5.2-rc',
            '0.7.0',
            '0.8.0-rc']


def test_ordering(version_list):
    l2 = copy.copy(version_list)
    random.seed(1)
    random.shuffle(l2)
    assert str(semver.versions(l2, stable=False)) != str(semver.versions(version_list, stable=False))
    assert str(sorted(semver.versions(l2, stable=False))) == str(semver.versions(version_list, stable=False))


def test_stable_only(version_list):
    assert semver.versions(version_list, stable=True) == semver.versions(["1.4.0", "1.6.0"])


def test_last_stable_version(version_list):
    assert str(semver.last_version(version_list, True)) == "1.6.0"


def test_last_unstable_version(version_list):
    assert str(semver.last_version(version_list, False)) == "1.7.0-rc"


def test_select_version(version_query):
    expected_results = [("==0.5.0", "0.5.0"),
                        (">=0.1.0", "0.7.0"),
                        ("<0.4.0", "0.3.0"),
                        (">=0.3.0,<0.6.0", "0.5.0"),
                        ([">=0.3.0", "<0.6.0"], "0.5.0"),
                        (">=0.1.0-", "0.8.0-rc"),
                        ("<0.4.0-", "0.3.2-rc"),
                        (">=0.3.0-,<0.6.0", "0.5.2-rc"),
                        (">=0.3.0,<0.6.0-", "0.5.2-rc"),
                        ("==10.0.0", 'None')]
    for query, expected in expected_results:
        assert str(semver.select_version(version_query, query)) == expected



import logging

import os.path
from collections import OrderedDict
from kpm.kubernetes import Kubernetes
from kpm.registry import Registry
from kpm.utils import colorize, mkdir_p
from kpm.display import print_deploy_result

logger = logging.getLogger(__name__)


def output_progress(kubsource, status, fmt="stdout"):
    if fmt == 'stdout':
        print " --> %s (%s): %s" % (kubsource.name, kubsource.kind, colorize(status))


def _process(package_name,
             version=None,
             dest="/tmp",
             namespace=None,
             force=False,
             dry=False,
             endpoint=None,
             action="create",
             fmt="stdout",
             proxy=None):

    registry = Registry(endpoint=endpoint)
    packages = registry.generate(package_name, namespace=namespace, version=version)
    dest = os.path.join(dest, package_name)

    if version:
        dest = os.path.join(dest, version)
    mkdir_p(dest)
    table = []
    results = []
    if fmt == "stdout":
        print "%s %s " % (action, package_name)
    i = 0
    for package in packages["deploy"]:
        i += 1
        pname = package["package"]
        version = package["version"]
        namespace = package["namespace"]
        if fmt == "stdout":
            print "\n %02d - %s:" % (i, package["package"])
        for resource in package["resources"]:
            body = resource["body"]
            endpoint = resource["endpoint"]
            # Use API instead of kubectl
            with open(os.path.join(dest, resource['file']), 'wb') as f:
                f.write(body)
            kubresource = Kubernetes(namespace=namespace, body=body, endpoint=endpoint, proxy=proxy)
            status = getattr(kubresource, action)(force=force, dry=dry)
            if fmt == "stdout":
                output_progress(kubresource, status)
            result_line = OrderedDict([("package", pname),
                                       ("version", version),
                                       ("kind", kubresource.kind),
                                       ("dry", dry),
                                       ("name", kubresource.name),
                                       ("namespace", kubresource.namespace),
                                       ("status", status)])

            if status != 'ok' and action == 'create':
                kubresource.wait(3)
            results.append(result_line)
            if fmt == "stdout":
                header = ["package", "version", "kind", "name",  "namespace", "status"]
                display_line = []
                for k in header:
                    display_line.append(result_line[k])
                table.append(display_line)
    if fmt == "stdout":
        print_deploy_result(table)
    return results


def deploy(*args, **kwargs):
    kwargs['action'] = 'create'
    return _process(*args, **kwargs)


def delete(*args, **kwargs):
    kwargs['action'] = 'delete'
    return _process(*args, **kwargs)



import pytest
import base64
import jinja2
import kpm.jinja_filters as filters


def test_get_hash(jinja_env):
    t = jinja_env.from_string("hello {{'titi' | get_hash}}")
    assert t.render() == "hello f7e79ca8eb0b31ee4d5d6c181416667ffee528ed"


def test_get_hash_md5(jinja_env):
    t = jinja_env.from_string("hello {{'titi' | get_hash('md5')}}")
    assert t.render() == "hello 5d933eef19aee7da192608de61b6c23d"


def test_get_hash_unknown(jinja_env):
    t = jinja_env.from_string("hello {{'titi' | get_hash('unknown')}}")
    with pytest.raises(ValueError):
        assert t.render() == "hello 5d933eef19aee7da192608de61b6c23d"


def test_b64encode(jinja_env):
    t = jinja_env.from_string("hello {{'titi' | b64encode}}")
    assert t.render() == "hello %s" % base64.b64encode('titi')


def test_b64decode(jinja_env):
    b64 = 'dGl0aQo='
    t = jinja_env.from_string("hello {{'%s' | b64decode}}" % b64)
    assert t.render() == "hello %s" % base64.b64decode(b64)


def test_b64chain(jinja_env):
    t = jinja_env.from_string("hello {{'titi' | b64encode | b64decode}}")
    assert t.render() == "hello titi"


# @TODO improve keygen tests
def test_gen_rsa(jinja_env):
    t = jinja_env.from_string("{{'rsa' | gen_privatekey}}")
    r = t.render()
    assert r.splitlines()[0] == "-----BEGIN RSA PRIVATE KEY-----"
    assert r.splitlines()[-1] == "-----END RSA PRIVATE KEY-----"


def test_gen_dsa(jinja_env):
    t = jinja_env.from_string("{{'dsa' | gen_privatekey}}")
    r = t.render()
    assert r.splitlines()[0] == "-----BEGIN DSA PRIVATE KEY-----"
    assert r.splitlines()[-1] == "-----END DSA PRIVATE KEY-----"


def test_gen_ecdsa(jinja_env):
    t = jinja_env.from_string("{{'ecdsa' | gen_privatekey}}")
    r = t.render()
    assert r.splitlines()[0] == "-----BEGIN EC PRIVATE KEY-----"
    assert r.splitlines()[-1] == "-----END EC PRIVATE KEY-----"


def test_gen_private_unknow(jinja_env):
    t = jinja_env.from_string("{{'unknown' | gen_privatekey}}")
    with pytest.raises(ValueError):
        assert t.render() == "raise error"


def test_rand_alphanum32(jinja_env):
    import re
    t = jinja_env.from_string("{{'32' | rand_alphanum}}")
    r = t.render()
    assert re.match("^([a-zA-Z0-9]{32})$", r) is not None


def test_rand_alpha32(jinja_env):
    import re
    t = jinja_env.from_string("{{'32' | rand_alpha}}")
    r = t.render()
    assert re.match("^([a-zA-Z]{32})$", r) is not None



import pytest
import kpm.utils
import os.path


def test_mkdirp_on_existing_dir(tmpdir):
    exists = str(tmpdir.mkdir("dir1"))
    kpm.utils.mkdir_p(exists)
    assert os.path.exists(exists)


def test_mkdirp(tmpdir):
    path = os.path.join(str(tmpdir), "new/directory/tocreate")
    kpm.utils.mkdir_p(path)
    assert os.path.exists(path)


def test_mkdirp_unauthorized(tmpdir):
    import os
    d = str(tmpdir.mkdir("dir2"))
    path = os.path.join(d, "directory/tocreate")
    os.chmod(d, 0)
    with pytest.raises(OSError):
        kpm.utils.mkdir_p(path)


def test_colorize():
    assert kpm.utils.colorize('ok') == "\x1b[32mok\x1b[0m"



import tempfile
import time
import logging
import json
import subprocess
import requests
from urlparse import urlparse

__all__ = ['Kubernetes', "get_endpoint"]


logger = logging.getLogger(__name__)


resource_endpoints = {
    "daemonsets": "apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
    "deployments": "apis/extensions/v1beta1/namespaces/{namespace}/deployments",
    "horizontalpodautoscalers": "apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
    "ingresses": "apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
    "jobs": "apis/extensions/v1beta1/namespaces/{namespace}/jobs",
    "namespaces": "api/v1/namespaces",
    "replicasets": "apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
    "persistentvolumes": "api/v1/namespaces/{namespace}/persistentvolumes",
    "persistentvolumeclaims": "api/v1/namespaces/{namespace}/persistentvolumeclaims",
    "services": "api/v1/namespaces/{namespace}/services",
    "serviceaccounts": "api/v1/namespaces/{namespace}/serviceaccounts",
    "secrets": "api/v1/namespaces/{namespace}/secrets",
    "configmaps": "api/v1/namespaces/{namespace}/configmaps",
    "replicationcontrollers": "api/v1/namespaces/{namespace}/replicationcontrollers",
    "pods": "api/v1/namespaces/{namespace}/pods",
}


resources_alias = {"ds": "daemonsets",
                   "hpa": "horizontalpodautoscalers",
                   "ing": "ingresses",
                   "ingress": "ingresses",
                   "ns": "namespaces",
                   "po": "pods",
                   "pv": "persistentvolumes",
                   "pvc": "persistentvolumeclaims",
                   "rc": "replicationcontrollers",
                   "svc": "services"}


def get_endpoint(kind):
    name = None
    if kind in resource_endpoints:
        name = kind
    elif kind in resources_alias:
        name = resources_alias[kind]
    elif kind + "s" in resource_endpoints:
        name = kind + "s"
    else:
        raise KeyError("Resource name %s not in [%s]" % (kind, resource_endpoints.keys()))
    return resource_endpoints[name]


class Kubernetes(object):
    def __init__(self,
                 namespace=None,
                 endpoint=None,
                 body=None,
                 proxy=None):

        self.proxy = None
        if endpoint is not None and endpoint[0] == "/":
            endpoint = endpoint[1:-1]
        self.endpoint = endpoint
        self.body = body
        self.obj = None
        self.protected = False
        self._resource_load()

        self.kind = self.obj['kind'].lower()
        self.name = self.obj['metadata']['name']
        self.kpmhash = self._get_kpmhash(self.obj)
        self.namespace = self._namespace(namespace)
        self.result = None
        if proxy:
            self.proxy = urlparse(proxy)

    def _resource_load(self):
        self.obj = json.loads(self.body)

        if 'annotations' in self.obj['metadata']:
            if ('kpm.protected' in self.obj['metadata']['annotations'] and
               self.obj['metadata']['annotations']['kpm.protected'] == 'true'):
                self.protected = True

    def _namespace(self, namespace=None):
        if namespace:
            return namespace
        elif 'namespace' in self.obj['metadata']:
            return self.obj['metadata']['namespace']
        else:
            return 'default'

    def create(self, force=False, dry=False):
        """
          - Check if resource name exists
          - if it exists check if the kpmhash is the same
          - if not the same delete the resource and recreate it
          - if force == true, delete the resource and recreate it
          - if doesnt exists create it
        """
        r = self.get()
        f = tempfile.NamedTemporaryFile()
        cmd = ['create', '-f', f.name]
        f.write(self.body)
        f.flush()
        if r is None:
            self._call(cmd, dry=dry)

            return 'created'
        elif (self.kpmhash is None or self._get_kpmhash(r) == self.kpmhash) and force is False:
            return 'ok'
        elif self._get_kpmhash(r) != self.kpmhash or force is True:
            if self.delete(dry=dry) == 'protected':
                return 'protected'
            self._call(cmd, dry=dry)
            return 'updated'

    def _get_kpmhash(self, r):
        if 'annotations' in r['metadata'] and 'kpm.hash' in r['metadata']['annotations']:
            return r['metadata']['annotations']['kpm.hash']
        else:
            return None

    def get(self):
        cmd = ['get', self.kind, self.name, '-o', 'json']
        try:
            self.result = json.loads(self._call(cmd))
            return self.result
        except subprocess.CalledProcessError:
            return None
        except (requests.exceptions.HTTPError) as e:
            if e.response.status_code == 404:
                return None
            else:
                raise e

    def delete(self, dry=False, **kwargs):
        cmd = ['delete', self.kind, self.name]
        if self.protected:
            return 'protected'
        r = self.get()
        if r is not None:
            self._call(cmd, dry=dry)
            return 'deleted'
        else:
            return 'absent'

    def wait(self, retries=3, seconds=1):
        r = 1
        time.sleep(seconds)
        obj = self.get()
        while (r < retries and obj is None):
            r += 1
            time.sleep(seconds)
            obj = self.get()
        return obj

    def exists(self):
        r = self.get()
        if r is None:
            return False
        else:
            return True

    def _call(self, cmd, dry=False):
        command = ['kubectl'] + cmd + ["--namespace", self.namespace]
        if not dry:
            if self.proxy is not None:
                return self._request(cmd[0])
            else:
                return subprocess.check_output(command, stderr=subprocess.STDOUT)
        else:
            return True

    def _request(self, method):
        if method == 'create':
            headers = {'Content-Type': 'application/json'}
            method = 'post'
            url = "%s/%s" % (self.proxy.geturl(), self.endpoint)
            return requests.post(url, data=self.body, headers=headers)
        else:
            url = "%s/%s/%s" % (self.proxy.geturl(), self.endpoint, self.name)
            query = getattr(requests, method)
            r = query(url)
            r.raise_for_status()
            return r.content



import os.path
import logging
import utils
import re

__all__ = ['new_package']

logger = logging.getLogger(__name__)

DIRECTORIES = ['templates']


MANIFEST = """---
package:
  name: {name}
  author: <author>
  version: 1.0.0
  description: {app}
  license: MIT

# Defaults variables
# i.e:
# variables:
#   namespace: kube-system
#   replicas: 1
#   image: "gcr.io/google_containers/heapster:v0.18.2"
#   svc_type: "NodePort"
variables: {{}}

# List the resources
# resources :
#  - file: nginx-rc.yaml # Template file , relative to ./templates
#    name: nginx         # kubernetes resource name
#    type: rc            # kubernetes resource type (ds,rc,svc,secret....)
#    sharded: no         # Optional: use the shards to generate this resource
#    patch:              # Optional: array of 'json-patch'
#      - {{op: replace, path: /metadata/labels/app-name, value: 'nginx'}}
resources:
  # - file: {app}-rc.yaml
  #   name: {app}
  #   type: rc

  # - file: {app}-svc.yaml
  #   name: {app}
  #   type: svc

# Shard list (optional)
# shards:
#   - name: shard-name     # will be append to the resource name
#     variables: {{}}        # Optional: apply vars  only to this shard
shards: []

# List de dependencies
# Special name '$self' to deploy current package.
# Useful to sort the dependencies
# i.e:
# deploy:
#   - name: postgresql
#   - name $self
deploy:
  - name: $self
"""

README = """
{name}
===========

# Install

kpm deploy {name}

"""


def new_package(name, dest=".", with_comments=False):
    _, app = name.split("/")
    path = os.path.join(dest, name)
    utils.mkdir_p(path)
    readme = open(os.path.join(path, 'README.md'), 'w')
    readme.write(README.format(name=name))
    readme.close()
    manifest = open(os.path.join(path, 'manifest.yaml'), 'w')
    if with_comments:
        m = MANIFEST
    else:
        m = re.sub(r'(?m)^#.*\n?', '', MANIFEST)
    manifest.write(m.format(app=app, name=name))
    manifest.close()
    for directory in DIRECTORIES:
        utils.mkdir_p(os.path.join(path, directory))



import os


class Config(object):
    """ Default configuration """
    DEBUG = False
    KUBE_APIMASTER = os.getenv('KUBE_APIMASTER', 'http://localhost:8001')
    KPM_REGISTRY_HOST = os.getenv('KPM_REGISTRY_HOSt', 'http://localhost:5000')
    KPM_BUILDER_HOST = os.getenv('KPM_BUILDER_HOST', 'http://localhost:5000')


class ProductionConfig(Config):
    """ Production configuration """
    KPM_URI = 'https://api.kpm.sh'


class DevelopmentConfig(Config):
    """ Development configuration """
    DEBUG = True
#    KPM_URI = 'https://api.kpm.sh'
    KPM_URI = os.getenv('KPM_URI', "http://localhost:5000")



import logging
import tarfile
import glob
import io

__all__ = ['pack_kub', 'unpack_kub', 'Package']

logger = logging.getLogger(__name__)

# 1. download the package
# 2. untar it to dest directory with the packagename_version/
# 3. open and read manifest.yaml
# 4. interate on deploy and download if not present (packagename_version/deps/deppackagename_version.kub) and extract
#    the packages to packagename_version/deps/deppackagename_version/.
# 5. interate on deploy and load manifest.yml of all packages
# 6. foreach manifest create the files to packagename_version/resources
# 7. foreach manifest check if resources exists are create it
#
#
#
AUTHORIZED_FILES = ["templates/*.yaml",
                    "templates/*.yml",
                    "templates/*.j2",
                    "templates/*.yaml.j2",
                    "templates/*.yml.j2",
                    "README.md",
                    "manifest.yaml",
                    "LICENSE",
                    "deps/*.kub"]


def pack_kub(kub):
    tar = tarfile.open(kub, "w:gz")
    for name in AUTHORIZED_FILES:
        for f in glob.glob(name):
            tar.add(f)
    tar.close()


def unpack_kub(kub, dest="."):
    tar = tarfile.open(kub, "r:gz")
    tar.extractall(dest)
    tar.close()


class Package(object):
    def __init__(self, blob=None):
        self.files = {}
        self.tar = None
        self.blob = None
        if blob is not None:
            self.load(blob)

    def load(self, blob):
        self.blob = blob
        self.tar = tarfile.open(fileobj=io.BytesIO(blob), mode='r:gz')
        for m in self.tar.getmembers():
            tf = self.tar.extractfile(m)
            if tf is not None:
                self.files[tf.name] = tf.read()

    def extract(self, dest):
        self.tar.extractall(dest)

    def pack(self, dest):
        f = open(dest, "wb")
        f.write(self.blob)
        f.close()

    def tree(self, directory=None):
        files = self.files.keys()
        files.sort()
        if directory is not None:
            filtered = [x for x in files if x.startswith(directory)]
        else:
            filtered = files
        return filtered

    def file(self, filename):
        return self.files[filename]

    @property
    def manifest(self):
        return self.files['manifest.yaml']



import string
import random
import hashlib
from base64 import b64decode, b64encode
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa, dsa


def get_hash(data, hashtype='sha1'):
    h = hashlib.new(hashtype)
    h.update(data)
    return h.hexdigest()


def rand_string(size=32, chars=(string.ascii_letters + string.digits)):
    size = int(size)
    return ''.join(random.choice(chars) for _ in range(size))


def rand_alphanum(size=32):
    size = int(size)
    return rand_string(size=size)


def rand_alpha(size=32):
    size = int(size)
    return rand_string(size=size, chars=string.ascii_letters)


def gen_private_ecdsa():
    from ecdsa import SigningKey
    sk = SigningKey.generate()
    return sk.to_pem()


def gen_private_rsa():
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption()
    )
    return pem


def gen_private_dsa():
    private_key = dsa.generate_private_key(
        key_size=1024,
        backend=default_backend()
    )
    pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption()
    )
    return pem


def gen_privatekey(keytype='rsa'):
    if keytype == "ecdsa":
        return gen_private_ecdsa()
    elif keytype == "rsa":
        return gen_private_rsa()
    elif keytype == "dsa":
        return gen_private_dsa()
    else:
        raise ValueError("Unknow private key type: %s" % keytype)


def filters():
    filters = {
        'get_hash': get_hash,
        'b64decode': b64decode,
        'b64encode': b64encode,
        'gen_privatekey': gen_privatekey,
        'rand_alphanum': rand_alphanum,
        'rand_alpha': rand_alpha
        }
    return filters



import json
import pytest
import requests
import requests_mock
from conftest import get_response
from kpm.kubernetes import get_endpoint, Kubernetes


NAMESPACE="testns"


def test_endpoints():
    a = get_endpoint("po")
    b = get_endpoint("pods")
    c = get_endpoint("pod")
    assert a == b
    assert b == c


def test_endpoints_missing():
    with pytest.raises(KeyError):
        get_endpoint("bad")


def test_protected_is_true_when_annoted(ns_resource):
    k = Kubernetes(body=ns_resource['body'])
    assert k.protected is True


def test_protected_is_false(svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k.protected is False


def test_force_namespace(ns_resource):
    k = Kubernetes(namespace="test", body=ns_resource['body'])
    assert k.namespace == "test"


def test__namespace(ns_resource):
    k = Kubernetes(body=ns_resource['body'])
    assert k._namespace() == "default"


def test__namespace_annotate(svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k._namespace() == "testns"


def test__namespace_params(svc_resource):
    k = Kubernetes(namespace="test", body=svc_resource['body'])
    assert k._namespace(k.namespace) == "test"


def test_load_obj(svc_resource):
    k = Kubernetes(namespace="test", body=svc_resource['body'])
    assert json.dumps(k.obj) == json.dumps(json.loads(svc_resource['body']))


def test_kind(svc_resource, rc_resource, ns_resource):
    ks = Kubernetes(body=svc_resource['body'])
    kr = Kubernetes(body=rc_resource['body'])
    kn = Kubernetes(body=ns_resource['body'])
    assert kr.kind == "replicationcontroller"
    assert kn.kind == "namespace"
    assert ks.kind == "service"


def test_get_hash(svc_resource):
    ks = Kubernetes(body=svc_resource['body'])
    kbody = json.loads(svc_resource['body'])
    assert ks.kpmhash == ks._get_kpmhash(kbody)
    assert ks.kpmhash == kbody['metadata']['annotations']['kpm.hash']


def test_get_empty(ns_resource):
    k = Kubernetes(body=ns_resource['body'])
    assert k.kpmhash is None


def test_check_cmd(svc_resource, subcall_cmd):
    k = Kubernetes(body=svc_resource['body'])
    assert k._call(["get", "svc", "kube-ui"]) == "kubectl get svc kube-ui --namespace testns"


def test_get_cmd_error(svc_resource, subcall_cmd_error):
    k = Kubernetes(body=svc_resource['body'])
    assert k.get() is None


def test_get_rc(subcall_get_assert, rc_resource):
    k = Kubernetes(body=rc_resource['body'])
    assert json.dumps(k.get()) == json.dumps(json.loads(get_response(k.name, k.kind)))


def test_get_svc(subcall_get_assert, svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert json.dumps(k.get()) == json.dumps(json.loads(get_response(k.name, k.kind)))


def test_get_ns(subcall_get_assert, ns_resource):
    k = Kubernetes(body=ns_resource['body'], namespace="testns")
    assert json.dumps(k.get()) == json.dumps(json.loads(get_response(k.name, k.kind)))


def test_delete_ns_protected(subcall_delete, ns_resource):
    k = Kubernetes(body=ns_resource['body'])
    assert k.delete() == "protected"


def test_delete_existing_resource(subcall_delete, svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k.delete() == "deleted"


def test_delete_non_existing_resource(subcall_cmd_error, svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k.delete() == "absent"


def test_exists_true(subcall_get_assert, svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k.exists() is True


def test_exists_false(subcall_cmd_error, svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k.exists() is False


def test_get_proxy(svc_resource):
    proxy = "http://localhost:8001"
    k = Kubernetes(body=svc_resource['body'], proxy=proxy, endpoint=svc_resource['endpoint'])
    url = "%s/%s/%s" % (proxy, svc_resource['endpoint'][1:-1], svc_resource['name'])
    url2 = "%s/%s/%s" % (k.proxy.geturl(), k.endpoint, k.name)
    assert url == url2
    with requests_mock.mock() as m:
        response = get_response(svc_resource["name"], svc_resource["kind"])
        m.get(url, text=response)
        assert json.dumps(k.get()) == json.dumps(json.loads(response))


def test_get_proxy_404(svc_resource):
    proxy = "http://localhost:8001"
    k = Kubernetes(body=svc_resource['body'], proxy=proxy, endpoint=svc_resource['endpoint'])
    url = "%s/%s/%s" % (proxy, svc_resource['endpoint'][1:-1], svc_resource['name'])
    with requests_mock.mock() as m:
        response = get_response(svc_resource["name"], svc_resource["kind"])
        m.get(url, text=response, status_code=404)
        assert k.get() is None


def test_get_proxy_500_raise(svc_resource):
    proxy = "http://localhost:8001"
    k = Kubernetes(body=svc_resource['body'], proxy=proxy, endpoint=svc_resource['endpoint'])
    url = "%s/%s/%s" % (proxy, svc_resource['endpoint'][1:-1], svc_resource['name'])
    with requests_mock.mock() as m:
        response = get_response(svc_resource["name"], svc_resource["kind"])
        m.get(url, text=response, status_code=500)
        with pytest.raises(requests.exceptions.HTTPError):
            k.get()


def test_call_dry(svc_resource):
    k = Kubernetes(body=svc_resource['body'])
    assert k._call(["get", "rc", "kube-ui"], dry=True)


def test_create_cmd(svc_resource, subcall_cmd, monkeypatch):
    def get(*args):
        return None
    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=svc_resource['body'])
    assert k.create() == "created"


def test_create_update(svc_resource, subcall_cmd, monkeypatch):
    def get(*args):
        svc2 = json.loads(svc_resource['body'])
        svc2['metadata']['annotations']['kpm.hash'] = 'dummy'
        return svc2

    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=svc_resource['body'])
    assert k.create() == "updated"


def test_create_ok(svc_resource, subcall_cmd, monkeypatch):
    def get(*args):
        svc2 = json.loads(svc_resource['body'])
        return svc2

    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=svc_resource['body'])
    assert k.create() == "ok"


def test_create_force(svc_resource, subcall_cmd, monkeypatch):
    def get(*args):
        svc2 = json.loads(svc_resource['body'])
        return svc2

    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=svc_resource['body'])
    assert k.create(force=True) == "updated"


def test_create_ok_nohash(ns_resource, subcall_cmd, monkeypatch):
    def get(*args):
        r = json.loads(ns_resource['body'])
        return r

    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=ns_resource['body'])
    assert k.create() == "ok"


def test_create_nohash(ns_resource, subcall_cmd, monkeypatch):
    def get(*args):
        return None
    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=ns_resource['body'])
    assert k.create() == "created"


def test_create_force_protected(ns_resource, subcall_cmd, monkeypatch):
    def get(*args):
        svc2 = json.loads(ns_resource['body'])
        return svc2

    monkeypatch.setattr("kpm.kubernetes.Kubernetes.get", get)
    k = Kubernetes(body=ns_resource['body'])
    assert k.create(force=True) == "protected"


def test_create_proxy(svc_resource):
    proxy = "http://localhost:8001"
    k = Kubernetes(body=svc_resource['body'], proxy=proxy, endpoint=svc_resource['endpoint'])
    url = "%s/%s" % (proxy, svc_resource['endpoint'][1:-1])
    url2 = "%s/%s" % (k.proxy.geturl(), k.endpoint)
    assert url == url2
    with requests_mock.mock() as m:
        response = get_response(svc_resource["name"], svc_resource["kind"])
        m.post(url, text=response)
        m.get(url + "/" + svc_resource['name'], status_code=404)
        assert k.create() == "created"


def test_wait(svc_resource, monkeypatch):
    import time
    monkeypatch.setattr(time, 'sleep', lambda s: None)
    proxy = "http://localhost:8001"
    k = Kubernetes(body=svc_resource['body'], proxy=proxy, endpoint=svc_resource['endpoint'])
    url = "%s/%s/%s" % (proxy, svc_resource['endpoint'][1:-1], svc_resource['name'])
    url2 = "%s/%s/%s" % (k.proxy.geturl(), k.endpoint, k.name)
    assert url == url2
    with requests_mock.mock() as m:
        response = get_response(svc_resource["name"], svc_resource["kind"])
        m.get(url, [{'status_code': 404}, {'text': response, 'status_code': 200}])
        assert json.dumps(k.wait()) == json.dumps(json.loads(response))
        assert m.call_count == 2



import pytest
import os.path
from kpm.auth import KpmAuth


def test_fake_home(fake_home):
    assert os.path.expanduser("~") == fake_home


def test_init_create_dir(fake_home):
    KpmAuth()
    assert os.path.exists(os.path.join(str(fake_home), ".kpm"))


def test_init_token_empty(fake_home):
    k = KpmAuth()
    assert os.path.exists(k.tokenfile) is False


def test_get_empty_token(fake_home):
    k = KpmAuth()
    assert k.token is None


def test_delete_empty_token(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    assert k.delete_token() is None


def test_delete_token(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    k.token = "titid"
    assert k.delete_token() == "titid"
    assert os.path.exists(k.tokenfile) is False


def test_create_token_value(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    k.token = "titic"
    assert k.token == "titic"


def test_create_token_file(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    k.token = "titib"
    assert os.path.exists(k.tokenfile) is True
    f = open(k.tokenfile, 'r')
    r = f.read()
    f.close()
    assert r == "titib"


def test_create_delete_get_token(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    k.token = "titia"
    assert k.token == "titia"
    k.delete_token()
    assert k.token is None


def test_get_token_from_file(fake_home):
    """ Should not fail if there is no token """
    k = KpmAuth()
    f = open(k.tokenfile, 'w')
    f.write("mytoken")
    f.close()
    assert k.token == "mytoken"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging

import socket
import datetime
import traceback as tb
import json


def _default_json_default(obj):
    """
    Coerce everything to strings.
    All objects representing time get output as ISO8601.
    """
    if (isinstance(obj, datetime.datetime) or
       isinstance(obj, datetime.date) or
       isinstance(obj, datetime.time)):
        return obj.isoformat()
    else:
        return str(obj)


class JsonFormatter(logging.Formatter):
    """
    A custom formatter to prepare logs to be in json format
    """

    def __init__(self,
                 fmt=None,
                 datefmt=None,
                 json_cls=None,
                 json_default=_default_json_default):
        """
        :param source_host: override source host name
        :param extra: provide extra fields always present in logs
        :param json_cls: JSON encoder to forward to json.dumps
        :param json_default: Default JSON representation for unknown types,
                             by default coerce everything to a string
        """

        if fmt is not None:
            self._fmt = json.loads(fmt)
        else:
            self._fmt = {}
        self.json_default = json_default
        self.json_cls = json_cls
        if 'extra' not in self._fmt:
            self.defaults = {}
        else:
            self.defaults = self._fmt['extra']
        if 'source_host' in self._fmt:
            self.source_host = self._fmt['source_host']
        else:
            try:
                self.source_host = socket.gethostname()
            except:
                self.source_host = ""

    def format(self, record):
        """
        Format a log record to JSON, if the message is a dict
        assume an empty message and use the dict as additional
        fields.
        """

        fields = record.__dict__.copy()
        if isinstance(record.msg, dict):
            fields.pop('msg')
            msg = ""
        else:
            msg = record.getMessage()

        if 'msg' in fields:
            fields.pop('msg')

        if 'exc_info' in fields:
            if fields['exc_info']:
                formatted = tb.format_exception(*fields['exc_info'])
                fields['exception'] = formatted
            fields.pop('exc_info')
        fields.pop('exc_text')
        fields.pop('args')
        fields.pop('created')
        fields.pop('filename')
        fields.pop('levelno')
        fields.pop('module')
        fields.pop('msecs')
        fields.pop('pathname')
        fields.pop('process')
        fields.pop('processName')
        fields.pop('relativeCreated')
        fields.pop('thread')
        fields.pop('threadName')
        if 'logstash' in fields:
            fields.pop('logstash')
        base_log = {'message': msg,
                    '@timestamp': datetime.datetime.utcnow(),
                    'source_host': self.source_host}
        base_log.update(fields)

        logr = self.defaults.copy()
        logr.update(base_log)

        return json.dumps(logr, default=self.json_default, cls=self.json_cls)


def init_logging(logger, loglevel="DEBUG",
                 fmt='[%(asctime)s: %(levelname)s][%(name)s:%(lineno)d] %(message)s'):

    logger.setLevel(logging.getLevelName(loglevel))
    json_fmt = JsonFormatter()
    f_formatter = json_fmt

    jsonh = logging.StreamHandler()
    jsonh.setLevel(logging.getLevelName(loglevel))
    jsonh.setFormatter(f_formatter)
    logger.addHandler(jsonh)



import json
import subprocess
import random
import logging


logger = logging.getLogger(__name__)


class KubernetesExec(object):
    def __init__(self, rcname, cmd='sh', namespace="default", container=None, kind="rc"):
        self.rcname = rcname
        self.namespace = namespace
        self.command = cmd
        self.kind = kind
        self.container = container

    def call(self, tty=True):
        rc = self._getrc()
        selector = self._getselector(rc)
        logger.info("selector: %s", selector)
        pods = self._getpods(selector)
        podname = random.choice(pods)['metadata']['name']
        cmd = ['exec', '--namespace', self.namespace, podname]
        if tty:
            cmd.append("-ti")
        if self.container is not None:
            cmd += ['-c', self.container]
        command = ['kubectl'] + cmd + ["--"] + self.command.split(" ")
        return subprocess.call(command)

    def _getpods(self, selector):
        cmd = ['get', "pods", "-l", selector, '-o', 'json']
        podslist = json.loads(self._call(cmd))
        pods = podslist['items']
        return pods

    def _getselector(self, rc):
        s = None
        items = rc['spec']['selector']
        if 'matchLabels' in items:
            items = items['matchLabels']
        for k, v in items.iteritems():
            if s is None:
                s = "%s=%s" % (k, v)
            else:
                s += ",%s=%s" % (k, v)
        return s

    def _getrc(self):
        cmd = ['get', self.kind, self.rcname, '-o', 'json']
        return (json.loads(self._call(cmd)))

    def _call(self, cmd, dry=False):
        command = ['kubectl'] + cmd + ["--namespace", self.namespace]
        return subprocess.check_output(command, stderr=subprocess.STDOUT)



import logging
import os.path
import yaml


__all__ = ['Manifest']
logger = logging.getLogger(__name__)

MANIFEST_FILE = "manifest.yaml"


class Manifest(dict):
    def __init__(self, package=None, path="."):
        if package is not None:
            self.update(yaml.safe_load(package.manifest))
            super(Manifest, self).__init__()

        elif path is not None:
            self.mfile = os.path.join(path, MANIFEST_FILE)
            self._load_yaml(self.mfile)

    def _load_yaml(self, mfile):
        try:
            y = yaml.safe_load(open(mfile, 'r'))
            self.update(y)
        except yaml.YAMLError, exc:
            print "Error in configuration file:"
            if hasattr(exc, 'problem_mark'):
                mark = exc.problem_mark
                print "Error position: (%s:%s)" % (mark.line+1, mark.column+1)
            raise exc

    @property
    def resources(self):
        return self.get("resources", [])

    @property
    def deploy(self):
        return self.get("deploy", [])

    @property
    def variables(self):
        return self.get("variables", {})

    @property
    def package(self):
        return self.get("package", {})

    @property
    def shards(self):
        return self.get("shards", [])

    def kubname(self):
        sp = self.package['name'].split('/')
        name = "%s_%s" % (sp[0], sp[1])
        return name

    def package_name(self):
        package = ("%s_%s" % (self.kubname(), self.package['version']))
        return package



import logging
from tabulate import tabulate
from kpm.utils import colorize

logger = logging.getLogger(__name__)


def print_packages(packages):
    header = ['app', 'version', 'downloads']
    table = []

    for p in packages:
        table.append([p['name'], p['version'],  str(p.get('downloads', '-'))])
    print tabulate(table, header)


def print_deploy_result(table):
    header = ["package", "version", "type", "name",  "namespace", "status"]
    print "\n"
    for r in table:
        status = r.pop()
        r.append(colorize(status))

    print tabulate(table, header)



import pytest
import os
from kpm.new import new_package


@pytest.fixture()
def home_dir(monkeypatch, fake_home):
    monkeypatch.chdir(str(fake_home))
    return str(fake_home)


@pytest.fixture()
def new(home_dir):
    new_package("organization/newpackage")


@pytest.fixture()
def new_with_comments(home_dir):
    new_package("organization/newpackage2", with_comments=True)


def test_directory(new):
    assert os.path.exists("organization/newpackage")


def test_directory_comments(new_with_comments):
    assert os.path.exists("organization/newpackage2")


def test_files_created(new):
    for f in ["templates", "manifest.yaml", "README.md"]:
        assert os.path.exists(os.path.join("organization/newpackage", f))


def test_load_manifest(new):
    import kpm.manifest
    m = kpm.manifest.Manifest(path="organization/newpackage")
    assert m.package["name"] == "organization/newpackage"
    assert m.deploy == [{'name': "$self"}]


def test_load_manifest_comments(new_with_comments):
    import kpm.manifest
    m = kpm.manifest.Manifest(path="organization/newpackage2")
    assert m.package["name"] == "organization/newpackage2"
    assert m.deploy == [{'name': "$self"}]



import json
import logging
from urlparse import urlparse, urljoin
import requests
import kpm
from kpm.auth import KpmAuth
from kpm.discovery import ishosted, discover_sources

__all__ = ['Registry']

logger = logging.getLogger(__name__)
DEFAULT_REGISTRY = "https://api.kpm.sh"
DEFAULT_STG_REGISTRY = "https://api-stg.kpm.sh"
API_PREFIX = '/api/v1'


class Registry(object):
    def __init__(self, endpoint=DEFAULT_REGISTRY):
        if endpoint is None:
            endpoint = DEFAULT_REGISTRY
        self.endpoint = urlparse(endpoint)
        self.auth = KpmAuth()
        self._headers = {'Content-Type': 'application/json',
                         'User-Agent': "kpmpy-cli: %s" % kpm.__version__}

    def _url(self, path):
        return urljoin(self.endpoint.geturl(), API_PREFIX + path)

    @property
    def headers(self):
        token = self.auth.token
        headers = {}
        headers.update(self._headers)
        if token is not None:
            headers['Authorization'] = token
        return headers

    def pull(self, name, version=None):
        if ishosted(name):
            sources = discover_sources(name)
            path = sources[0]
        else:
            organization, name = name.split("/")
            path = self._url("/packages/%s/%s/pull" % (organization, name))
        params = {"version": version}
        r = requests.get(path, params=params, headers=self.headers)
        r.raise_for_status()
        return r.content

    def list_packages(self, user=None, organization=None):
        path = "/packages"
        params = {}
        if user:
            params['username'] = user
        if organization:
            params["organization"] = organization
        r = requests.get(self._url(path), params=params, headers=self.headers)
        r.raise_for_status()
        return r.json()

    def generate(self, name, namespace=None, variables=None, version=None, tarball=False):
        path = "/packages/%s/generate" % name
        params = {}
        if tarball:
            params['tarball'] = 'true'
        if version:
            params['version'] = version
        if namespace:
            params['namespace'] = namespace
        if variables:
            params['variables'] = variables
        r = requests.get(self._url(path), params=params, headers=self.headers)
        r.raise_for_status()
        return r.json()

    def push(self, name, body, force=False):
        organization, pname = name.split("/")
        body['name'] = pname
        body['organization'] = organization
        body['package'] = name
        path = "/packages/%s/%s" % (organization, pname)
        r = requests.post(self._url(path),
                          params={"force": str(force).lower()},
                          data=json.dumps(body), headers=self.headers)
        r.raise_for_status()
        return r.json()

    def login(self, username, password):
        path = "/users/login"
        self.auth.delete_token()
        r = requests.post(self._url(path),
                          params={"user[username]": username,
                                  "user[password]": password},
                          headers=self.headers)
        r.raise_for_status()
        result = r.json()
        self.auth.token = result['token']
        return result

    def signup(self, username, password, password_confirmation, email):
        path = "/users"
        self.auth.delete_token()
        r = requests.post(self._url(path),
                          params={"user[username]": username,
                                  "user[password]": password,
                                  "user[password_confirmation]": password_confirmation,
                                  "user[email]": email,
                                  },
                          headers=self.headers)
        r.raise_for_status()
        result = r.json()
        self.auth.token = result['token']
        return result

    def delete_package(self, name, version=None):
        organization, name = name.split("/")
        path = "/packages/%s/%s" % (organization, name)
        params = {}
        if version:
            params['version'] = version
        r = requests.delete(self._url(path), params=params, headers=self.headers)
        r.raise_for_status()
        return True



from semantic_version import Version, Spec


def versions(versions_str, stable=False):
    if stable:
        return [Version(x) for x in versions_str if len(x.split("-")) == 1]
    else:
        return [Version(x) for x in versions_str]


def last_version(versions_str, stable=False):
    return sorted(versions(versions_str, stable))[-1]


def select_version(versions_str, query, stable=False):
    if isinstance(query, str):
        query = query.split(",")
    query = [x.replace(" ", "") for x in query]
    stable = True
    if "-" in str(query):
        stable = False
    spec = Spec(*query)
    return spec.select(versions(versions_str, stable))



import copy
import hashlib
import tarfile
import shutil
import logging
import os.path
import jinja2
import yaml
import io
import tempfile
import jsonpatch
import json
from collections import OrderedDict
import kpm.registry as registry
import kpm.packager as packager
import kpm.manifest as manifest
import kpm.jinja_filters as jinja_filters
from kpm.kubernetes import get_endpoint
from kpm.utils import mkdir_p, convert_utf8


# __all__ = ['Kub']

logger = logging.getLogger(__name__)


_mapping_tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG


def dict_representer(dumper, data):
    return dumper.represent_dict(data.iteritems())


def dict_constructor(loader, node):
    return OrderedDict(loader.construct_pairs(node))


yaml.add_representer(OrderedDict, dict_representer)
yaml.add_constructor(_mapping_tag, dict_constructor)

jinja_env = jinja2.Environment()
jinja_env.filters.update(jinja_filters.filters())


class Kub(object):
    def __init__(self, name, version=None, resources=[], variables={}, shards=[], namespace=None, endpoint=None):
        self.name = name
        self.endpoint = endpoint
        self._dependencies = None
        self._resources = None
        self._deploy_resources = resources
        self._deploy_shards = shards
        self.namespace = namespace
        self._registry = registry.Registry(endpoint=endpoint)
        result = self._registry.pull(self.name, version)
        self.package = packager.Package(result)

        self.manifest = manifest.Manifest(self.package)
        self.version = self.manifest.package['version']
        self.author = self.manifest.package['author']
        self.description = self.manifest.package['description']
        self.deploy = self.manifest.deploy
        self.variables = copy.deepcopy(self.manifest.variables)
        if self.namespace:
            variables["namespace"] = self.namespace
        self.variables.update(variables)

    def __unicode__(self):
        return ("(<{class_name}({name}=={version})>".format(class_name=self.__class__.__name__,
                                                            name=self.name, version=self.version))

    def __str__(self):
        return unicode(self).encode('utf-8')

    def __repr__(self):
        return unicode(self).encode('utf-8')

    def _create_namespaces(self, resources):
        # @TODO create namespaces for all manifests
        if self.namespace:
            ns = self.create_namespace(self.namespace)
            resources[ns['file']] = ns
        return resources

    def create_namespace(self, namespace):
        value = {"apiVersion": "v1",
                 "kind": "Namespace",
                 "metadata": {"name": namespace}}

        resource = {"file": "%s-ns.yaml" % namespace,
                    "value": value,
                    "name": namespace,
                    "generated": True,
                    "order": -1,
                    "hash": False,
                    "protected": True,
                    "patch": [],
                    "variables": {},
                    "type": "namespace"}
        return resource

    @property
    def dependencies(self):
        if self._dependencies is None:
            self._fetch_deps()
        return self._dependencies

    def _append_patch(self, resources={}):
        index = 0

        for resource in self.manifest.resources:
            index += 1
            resources[resource['file']] = resource
            resource["order"] = index
            if 'protected' not in resource:
                resource["protected"] = False
            if 'patch' not in resource:
                resource['patch'] = []

        for resource in self._deploy_resources:
            if 'patch' in resource and len(resource['patch']) > 0:
                resources[resource['file']]["patch"] += resource['patch']

        return resources

    @property
    def shards(self):
        shards = self.manifest.shards
        if len(self._deploy_shards):
            shards = self._deploy_shards
        return shards

    def _generate_shards(self, resources):
        if not len(self.shards):
            return resources
        sharded = {}
        to_remove = []
        index = 0
        for _, resource in resources.iteritems():
            index += 1
            resource['order'] = index
            if 'sharded' in resource and resource['sharded'] is True:
                for shard in self.shards:
                    shard_vars = shard.get('variables', {})
                    shard_vars.update({"name": shard['name']})

                    r = {"file": "%s-%s.yaml" % (os.path.splitext(resource['file'])[0], shard['name']),
                         "order": index,
                         "protected": False,
                         "template": resource['file'],
                         "variables": shard_vars,
                         "patch": resource['patch'] + shard.get('patch', []),
                         "name": "%s-%s" % (resource['name'], shard['name']),
                         "type": resource['type']}
                    sharded[r['file']] = r
                    index += 1
                to_remove.append(resource['file'])
        map(resources.pop, to_remove)
        resources.update(sharded)
        return resources

    def _default_patch(self, resources):
        for _, resource in resources.iteritems():
            patch = [
                {"op": "replace",
                 "path": "/metadata/name",
                 "value": resource['name']},
            ]
            if 'patch' not in resource:
                resource['patch'] = []
            resource['patch'] += patch
        return resources

    def _resolve_jinja(self, resources, from_value=False):
        for _, resource in resources.iteritems():
            if 'template' in resource:
                tpl_file = resource['template']
            else:
                tpl_file = resource['file']
            if from_value or resource.get('generated', False) is True:
                val = yaml.safe_dump(convert_utf8(resource['value']))
            else:
                val = self.package.files[os.path.join('templates', tpl_file)]
            template = jinja_env.from_string(val)
            variables = copy.deepcopy(self.variables)
            if 'variables' in resource:
                variables.update(resource['variables'])
            if len(self.shards):
                variables['kpmshards'] = self.shards
            t = template.render(variables)
            resource['value'] = yaml.load(t)
        return resources

    def _apply_patches(self, resources):
        for _, resource in resources.iteritems():
            if self.namespace:
                if 'namespace' in resource['value']['metadata']:
                    op = 'replace'
                else:
                    op = 'add'
                resource['patch'].append({"op": op, "path": "/metadata/namespace", "value": self.namespace})

            if len(resource['patch']):
                patch = jsonpatch.JsonPatch(resource['patch'])
                result = patch.apply(resource['value'])
                resource['value'] = result
        return resources

    def resources(self):
        if self._resources is None:
            self._resources = OrderedDict()
            resources = self._resources
            resources = self._create_namespaces(resources)
            resources = self._append_patch(resources)
            resources = self._generate_shards(resources)
            resources = self._default_patch(resources)
            resources = self._resolve_jinja(resources)
            resources = self._apply_patches(resources)
            resources = self._resolve_jinja(resources, True)
        return self._resources

    def prepare_resources(self, dest="/tmp", index=0):
        for _, resource in self.resources().iteritems():
            index += 1
            path = os.path.join(dest, "%02d_%s_%s" % (index,
                                                      self.version,
                                                      resource['file']))
            f = open(path, 'w')
            f.write(yaml.safe_dump(convert_utf8(resource['value'])))
            resource['filepath'] = f.name
            f.close()
        return index

    def make_tarfile(self, source_dir):
        output = io.BytesIO()
        with tarfile.open(fileobj=output, mode="w:gz") as tar:
            tar.add(source_dir, arcname=os.path.basename(source_dir))
        return output

    def build_tar(self, dest="/tmp"):
        tempdir = tempfile.mkdtemp()
        dest = os.path.join(tempdir, self.manifest.package_name())
        mkdir_p(dest)
        index = 0
        for kub in self.dependencies:
            index = kub.prepare_resources(dest, index)

        package_json = self.build()
        with open(os.path.join(dest, ".package.json"), mode="w") as f:
            f.write(json.dumps(package_json))

        tar = self.make_tarfile(dest)
        tar.flush()
        tar.seek(0)
        shutil.rmtree(tempdir)
        return tar.read()

    def build(self):
        result = []
        for kub in self.dependencies:
            kubresources = OrderedDict([("package",  kub.name),
                                        ("version", kub.version),
                                        ("namespace", kub.namespace),
                                        ("resources", [])])
            for _, resource in kub.resources().iteritems():
                sha = None
                if 'annotations' not in resource['value']['metadata']:
                    resource['value']['metadata']['annotations'] = {}
                if resource.get('hash', True):
                    sha = hashlib.sha256(json.dumps(resource['value'])).hexdigest()
                    resource['value']['metadata']['annotations']['kpm.hash'] = sha
                resource['value']['metadata']['annotations']['kpm.version'] = kub.version
                resource['value']['metadata']['annotations']['kpm.package'] = kub.name
                resource['value']['metadata']['annotations']['kpm.parent'] = self.name
                resource['value']['metadata']['annotations']['kpm.protected'] = str(resource['protected']).lower()

                kubresources['resources'].\
                    append(OrderedDict({"file": resource['file'],
                                        "hash": sha,
                                        "protected": resource['protected'],
                                        "name": resource['name'],
                                        "kind": resource['value']['kind'].lower(),
                                        "endpoint": get_endpoint(
                                            resource['value']['kind'].lower()).
                                        format(namespace=self.namespace),
                                        "body": json.dumps(resource['value'])}))

            result.append(kubresources)
        return {"deploy": result,
                "package": {"name": self.name,
                            "version": self.version}}

    def _fetch_deps(self):
        self._dependencies = []
        for dep in self.manifest.deploy:
            if dep['name'] != '$self':
                variables = dep.get('variables', {})
                variables['kpmparent'] = {'name': self.name,
                                          'shards': self.shards,
                                          'variables': self.variables}
                kub = Kub(dep['name'],
                          endpoint=self.endpoint,
                          version=dep.get('version', None),
                          variables=variables,
                          shards=dep.get('shards', []),
                          resources=dep.get('resources', []),
                          namespace=self.namespace)
                self._dependencies.append(kub)
            else:
                self._dependencies.append(self)



import os
from flask import Flask
import flask.ext.cors
from kpm.loghandler import init_logging


def create_app():
    app = Flask(__name__)
    flask.ext.cors.CORS(app)
    setting = os.getenv('APP_ENV', "development")

    if setting != 'production':
        app.config.from_object('kpm.api.config.DevelopmentConfig')
    else:
        app.config.from_object('kpm.api.config.ProductionConfig')
    from kpm.api.builder import builder_app
    from kpm.api.info import info_app
    from kpm.api.registry import registry_app
    app.register_blueprint(builder_app)
    app.register_blueprint(info_app)
    app.register_blueprint(registry_app)
    init_logging(app.logger, loglevel='INFO')
    app.logger.info("Start service")
    return app


if __name__ == "__main__":
    app = create_app()
    app.run(host='0.0.0.0')



import json
import subprocess
import pytest
import os
import pytest
import requests
from kpm.utils import colorize
from kpm.command import (new,
                         show,
                         install,
                         remove,
                         pull,
                         push,
                         list_packages,
                         login,
                         logout,
                         delete_package,
                         get_parser)


# 1. signup with uniq-test-id account
# 2. push package with test-id as organization
# 3. logout -> check ~/.kpm/auth is not exists
# 4. try to push -> failed
# 5. login
# 7. list -u test-id
# -  delete package
# -  list -u test-id
# -  push again
# -  pull
# -  install package
# -  remove package
# 9. show test-1/package == manifest
# 10. show --tree
# 11. show file template/kube-ui-svc.yaml
PROJECT_DIR=os.getcwd()


@pytest.fixture(scope="module")
def project_dir():
    return PROJECT_DIR

@pytest.fixture(scope="module")
def testid():
    import random
    import uuid
    a = list(uuid.uuid1().hex)
    random.shuffle(a)
    return "".join(a)[0:8]


@pytest.fixture(scope="module")
def parser():
    return get_parser()


@pytest.fixture(scope="module")
def pkgname(testid):
    return "org-%s/pkg-%s" % (testid, testid)


@pytest.fixture(scope="module")
def cli_home(tmpdir_factory):
    home = str(tmpdir_factory.mktemp('home'))
    os.environ['HOME'] = home
    return home


@pytest.fixture(scope="module")
def package_home(pkgname, cli_home):
    import shutil

    package_home = str(cli_home) + "/kube-ui"
    if os.path.exists(package_home) is False:
        manifestpath = package_home + "/manifest.yaml"
        shutil.copytree("tests/data/kube-ui", package_home)
        # Read in the file
        filedata = None
        with open(manifestpath, 'r') as f:
            filedata = f.read()
        filedata = filedata.replace('kube-system/kube-ui', pkgname)
        with open(manifestpath, 'w') as file:
            file.write(filedata)

    return package_home


@pytest.fixture(scope="module")
def email(testid):
    return "%s@kubespray.io" % testid


@pytest.fixture(scope="module")
def user(testid):
    return "user-%s" % testid


@pytest.fixture(scope="module")
def password(testid):
    return "%s" % testid


@pytest.fixture(scope="module")
def api_stg():
    host = os.environ.get("KPM_API", "https://api-stg.kpm.sh")
    return ["-H", host]


@pytest.mark.cli
@pytest.mark.last
def test_signup(user, email, password, parser, api_stg, package_home, capsys):
    cmd = "login -u %s -p %s -e %s --signup" % (user, password, email)
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    response = ' >>> Registration complete\n'
    out, err = capsys.readouterr()
    assert out == response


@pytest.mark.cli
@pytest.mark.last
def test_push_1(package_home, parser, monkeypatch, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "push"
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    assert out == "package: %s (1.0.1) pushed\n" % pkgname


@pytest.mark.cli
@pytest.mark.last
def test_push_1_existing(package_home, parser, monkeypatch, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "push"
    args = parser.parse_args(cmd.split(" ") + api_stg)
    with pytest.raises(requests.HTTPError):
        args.func(args)


@pytest.mark.cli
@pytest.mark.last
def test_push_1_force(package_home, parser, monkeypatch, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "push --force"
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    assert out == "package: %s (1.0.1) pushed\n" % pkgname


@pytest.mark.cli
@pytest.mark.last
def test_list_packages_ok(package_home, user, parser, pkgname, api_stg, capsys):
    cmd = "list -u %s" % user
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    response = """app                        version      downloads
-------------------------  ---------  -----------
{pkgname}  1.0.1                0\n""".format(pkgname=pkgname)
    assert out == response


@pytest.mark.cli
@pytest.mark.last
def test_show(package_home, parser, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "show %s" % pkgname
    manifestpath = package_home + "/manifest.yaml"
    # Read in the file
    filedata = None
    with open(manifestpath, 'r') as f:
        filedata = f.read()

    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)

    out, err = capsys.readouterr()
    assert out == filedata + "\n"


@pytest.mark.cli
@pytest.mark.last
def test_show_tree(package_home, parser, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "show --tree %s" % pkgname
    manifestpath = package_home + "/manifest.yaml"
    # Read in the file
    filedata = None
    with open(manifestpath, 'r') as f:
        filedata = f.read()

    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)

    out, err = capsys.readouterr()
    assert out == """README.md
manifest.yaml
templates/kube-ui-rc.yaml
templates/kube-ui-svc.yaml\n"""


@pytest.mark.cli
@pytest.mark.last
def test_show_file(package_home, parser, pkgname, api_stg, capsys):
    os.chdir(package_home)
    cmd = "show -f templates/kube-ui-rc.yaml %s" % pkgname
    path = package_home + "/templates/kube-ui-rc.yaml"
    # Read in the file
    filedata = None
    with open(path, 'r') as f:
        filedata = f.read()

    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)

    out, err = capsys.readouterr()
    assert out == filedata + "\n"


@pytest.mark.cli
@pytest.mark.last
def test_logout(cli_home, parser, api_stg, capsys):
    os.chdir(str(cli_home))
    assert os.path.exists(".kpm/auth_token")
    cmd = "logout"
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    assert out == ' >>> Logout complete\n'
    assert not os.path.exists(".kpm/auth_token")


@pytest.mark.cli
@pytest.mark.last
def test_login(cli_home, user, password, parser, api_stg, capsys):
    os.chdir(str(cli_home))
    cmd = "login -u %s -p %s" % (user, password)
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    assert out == ' >>> Login succeeded\n'
    assert os.path.exists(".kpm/auth_token")


@pytest.mark.cli
@pytest.mark.last
def test_pull(package_home, parser, pkgname, api_stg):
    cmd = "pull %s" % pkgname
    os.chdir(str(package_home))
    manifestpath = package_home + "/manifest.yaml"
    # Read in the file
    filedata = None
    with open(manifestpath, 'r') as f:
        filedata = f.read()

    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    pulldata = None
    with open(package_home + ("/%s_1.0.1/manifest.yaml" % pkgname.replace("/", "_")), 'r') as f:
        pulldata = f.read()
    assert pulldata == filedata


@pytest.mark.cli
@pytest.mark.last
def test_new(cli_home, parser, pkgname, api_stg):
    os.chdir(str(cli_home))
    cmd = "new t2/test"
    args = parser.parse_args(cmd.split(" "))
    args.func(args)
    assert os.path.exists("t2/test/manifest.yaml")


@pytest.mark.cli
@pytest.mark.last
def test_deploy(project_dir, subcall_all, user, parser, pkgname, api_stg, capsys):
    os.chdir(project_dir)
    cmd = "deploy %s --namespace=testns" % pkgname
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    response = """create {pkgname} \n
 01 - {pkgname}:
 --> testns (namespace): {ok}
 --> kube-ui (replicationcontroller): {ok}
 --> kube-ui (service): {updated}


package                    version    type                   name     namespace    status
-------------------------  ---------  ---------------------  -------  -----------  --------
{pkgname}  1.0.1      namespace              testns   testns       {ok}
{pkgname}  1.0.1      replicationcontroller  kube-ui  testns       {ok}
{pkgname}  1.0.1      service                kube-ui  testns       {updated}\n""".format(pkgname=pkgname,
                                                                                         ok=colorize('ok'),
                                                                                         updated=colorize('updated'))

    # with open("/tmp/r", "w") as f:
    #     f.write(out)
    # with open("/tmp/r2", "w") as f:
    #     f.write(response)

    assert out == response


@pytest.mark.cli
@pytest.mark.last
def test_remove(project_dir, subcall_all, user, parser, pkgname, api_stg, capsys):
    os.chdir(project_dir)
    cmd = "remove %s --namespace=testns" % pkgname
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    response = """delete {pkgname} \n
 01 - {pkgname}:
 --> testns (namespace): {protected}
 --> kube-ui (replicationcontroller): {deleted}
 --> kube-ui (service): {deleted}


package                    version    type                   name     namespace    status
-------------------------  ---------  ---------------------  -------  -----------  ---------
{pkgname}  1.0.1      namespace              testns   testns       {protected}
{pkgname}  1.0.1      replicationcontroller  kube-ui  testns       {deleted}
{pkgname}  1.0.1      service                kube-ui  testns       {deleted}\n""".format(pkgname=pkgname,
                                                                                         protected=colorize('protected'),
                                                                                         deleted=colorize('deleted'))
    assert out == response


@pytest.mark.cli
@pytest.mark.last
def test_delete_package(package_home, user, parser, pkgname, api_stg, capsys):
    cmd = "delete-package %s" % pkgname
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    assert out == "Package %s deleted\n" % pkgname


@pytest.mark.cli
@pytest.mark.last
def test_list_packages_missing(package_home, user, parser, pkgname, api_stg, capsys):
    cmd = "list -u %s" % user
    args = parser.parse_args(cmd.split(" ") + api_stg)
    args.func(args)
    out, err = capsys.readouterr()
    response = """app    version    downloads
-----  ---------  -----------\n"""

    assert out == response



import pytest
from kpm.manifest import Manifest


@pytest.fixture(scope="module")
def manifest(kubeui_package):
    return Manifest(kubeui_package)


@pytest.fixture(scope="module")
def empty_manifest():
    return Manifest(None, None)


def test_empty_resources(empty_manifest):
    assert empty_manifest.resources == []


def test_empty_variables(empty_manifest):
    assert empty_manifest.variables == {}


def test_empty_package(empty_manifest):
    assert empty_manifest.package == {}


def test_empty_shards(empty_manifest):
    assert empty_manifest.shards == []


def test_empty_deploy(empty_manifest):
    assert empty_manifest.deploy == []


def test_package_name(manifest):
    assert manifest.package_name() == "kube-system_kube-ui_1.0.1"


def test_kubename(manifest):
    assert manifest.kubname() == "kube-system_kube-ui"


def test_load_from_path(manifest):
    m = Manifest(path="tests/data/kube-ui")
    assert m == manifest


def test_load_bad_manifest(manifest):
    import yaml
    with pytest.raises(yaml.YAMLError):
        Manifest(path="tests/data/bad_manifest")



import errno
import os
import collections
from termcolor import colored


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def colorize(status):
    msg = {'ok': 'green',
           'created': 'yellow',
           'updated': 'yellow',
           'absent': 'green',
           'deleted': 'red',
           'protected': 'blue'}
    return colored(status, msg[status])


def convert_utf8(data):
    if isinstance(data, basestring):
        return str(data)
    elif isinstance(data, collections.Mapping):
        return dict(map(convert_utf8, data.iteritems()))
    elif isinstance(data, collections.Iterable):
        return type(data)(map(convert_utf8, data))
    else:
        return data



class ApiException(Exception):
    status_code = 500
    errorcode = "internal-error"

    def __init__(self, message, payload=None):
        Exception.__init__(self)
        self.payload = dict(payload or ())
        self.message = message

    def to_dict(self):
        r = {"code": self.errorcode,
             "message": self.message,
             "details": self.payload}
        return r


class InvalidUsage(ApiException):
    status_code = 400
    errorcode = "invalid-usage"


class InvalidVersion(ApiException):
    status_code = 422
    errorcode = "invalid-version"


class PackageAlreadyExists(ApiException):
    status_code = 409
    errorcode = "package-exists"


class PackageNotFound(ApiException):
    status_code = 404
    errorcode = "package-not-found"


class PackageVersionNotFound(ApiException):
    status_code = 404
    errorcode = "package-version-not-found"



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'futures',
    'requests',
    'pyyaml',
    'jinja2',
    'jsonpatch',
    'tabulate',
    'termcolor',
    'python-etcd',
    'semantic_version',
    'flask',
    'Flask>=0.10.1',
    'flask-cors',
    'ecdsa',
    'cryptography'
]

test_requirements = [
    "pytest",
    "pytest-cov",
    'pytest-flask',
    "pytest-ordering",
    "requests-mock"
]

setup(
    name='kpm',
    version='0.15.0',
    description="KPM cli",
    long_description=readme + '\n\n' + history,
    author="Antoine Legrand",
    author_email='2t.antoine@gmail.com',
    url='https://github.com/kubespray/kpm',
    packages=[
        'kpm',
        'kpm.api'
    ],
    scripts=[
        'bin/kpm'
    ],
    package_dir={'kpm':
                 'kpm'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache License version 2",
    zip_safe=False,
    keywords=['kpm', 'kpmpy', 'kubernetes'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



import pytest
import requests
import requests_mock
import json
from collections import OrderedDict
from kpm.deploy import deploy, _process, delete


@pytest.fixture(autouse=True)
def nosleep(monkeypatch):
    import time
    monkeypatch.setattr(time, 'sleep', lambda s: None)


@pytest.fixture()
def deploy_result():
    return [OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'namespace'), ('dry', False), ('name', u'testns'), ('namespace', u'testns'), ('status', 'ok')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'replicationcontroller'), ('dry', False), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'ok')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'service'), ('dry', False), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'updated')])]


@pytest.fixture()
def deploy_dry_result():
    return [OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'namespace'), ('dry', True), ('name', u'testns'), ('namespace', u'testns'), ('status', 'ok')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'replicationcontroller'), ('dry', True), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'ok')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'service'), ('dry', True), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'updated')])]



@pytest.fixture()
def remove_result():
    return [OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'namespace'), ('dry', False), ('name', u'testns'), ('namespace', u'testns'), ('status', 'protected')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'replicationcontroller'), ('dry', False), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'deleted')]), OrderedDict([('package', u'ant31/kube-ui'), ('version', u'1.0.1'), ('kind', u'service'), ('dry', False), ('name', u'kube-ui'), ('namespace', u'testns'), ('status', 'deleted')])]


def test_deploy(deploy_json, deploy_result, subcall_all):
    url = "https://api.kpm.sh/api/v1/packages/%s/%s/generate" % ("ant31", "kube-ui")
    with requests_mock.mock() as m:
        m.get(url, text=deploy_json)
        r = deploy("ant31/kube-ui",
                   version="1.0.1",
                   namespace=None,
                   force=False,
                   dry=False,
                   endpoint=None)
        assert json.dumps(r) == json.dumps(deploy_result)


def test_remove(deploy_json, remove_result, subcall_all):
    url = "https://api.kpm.sh/api/v1/packages/%s/%s/generate" % ("ant31", "kube-ui")
    with requests_mock.mock() as m:
        m.get(url, text=deploy_json)
        r = delete("ant31/kube-ui",
                   version=None,
                   namespace=None,
                   force=False,
                   dry=False,
                   endpoint=None)
        print r
        assert json.dumps(r) == json.dumps(remove_result)


def test_deploy_dry(deploy_json, deploy_dry_result, subcall_all):
    url = "https://api.kpm.sh/api/v1/packages/%s/%s/generate" % ("ant31", "kube-ui")
    with requests_mock.mock() as m:
        m.get(url, text=deploy_json)
        r = deploy("ant31/kube-ui",
                   version=None,
                   namespace=None,
                   force=False,
                   dry=True,
                   endpoint=None)
        assert json.dumps(r) == json.dumps(deploy_dry_result)



import re
import requests
from HTMLParser import HTMLParser


package_regexp = "(.+?)/(.+)"


class MetaHTMLParser(HTMLParser):
    def __init__(self, variables=None):
        self.meta = {}
        if variables is None:
            variables = {}
        self.variables = variables
        HTMLParser.__init__(self)

    def replace_values(self, s):
        for k, v in self.variables.iteritems():
            s = s.replace("{%s}" % k, v)
        return s

    def handle_starttag(self, tag, attrs):
        if tag == "meta":
            d = dict(attrs)
            if 'name' in d and d['name'] == 'kpm-package':
                name, source = d['content'].split(" ")
                name = self.replace_values(name)
                source = self.replace_values(source)
                if name not in self.meta:
                    self.meta[name] = []
                self.meta[name].append(source)


def ishosted(package):
    m = re.search(package_regexp, package)
    host = m.group(1)
    if "." in host:
        return True
    else:
        return False


def discover_sources(package, secure=False):
    m = re.search(package_regexp, package)
    host, name = (m.group(1), m.group(2))
    schemes = ["https://", "http://"]
    for scheme in schemes:
        url = scheme + host
        try:
            r = requests.get(url, params={"kpm-discovery": 1})
        except requests.ConnectionError as e:
            if scheme == "https://" and not secure:
                continue
            else:
                raise e

        r.raise_for_status()
        p = MetaHTMLParser({'name': name})
        p.feed(r.content)
    if package in p.meta:
        return p.meta[package]
    else:
        return None



import pytest
from kpm.display import print_packages, print_deploy_result


@pytest.fixture()
def package_list():
    h = [{"name": "o1/p1",
          "version": "1.4.0",
          "downloads": 45,
          "available_versions": ["1.3.0", "1.2.0"]},
         {"name": "o1/p2",
          "version": "1.4.0",
          "downloads": 45,
          "available_versions": ["1.3.0", "1.2.0"]}]
    return h


@pytest.fixture()
def deploy_result():
    from collections import OrderedDict
    h = [OrderedDict([("name", "o1/p1"),
                      ("version", "1.4.0"),
                      ("type", "replicationcontroller"),
                      ("name", "p1"),
                      ("namespace", "testns"),
                      ("status", "ok")]).values(),
         OrderedDict([("name", "o1/p1"),
                      ("version", "1.4.0"),
                      ("type", "svc"),
                      ("name", "p1"),
                      ("namespace", "testns"),
                      ("status", "updated")]).values(),

         ]
    return h


def test_empty_list(capsys):
    print_packages([])
    out, err = capsys.readouterr()
    res = unicode("\n".join(["app    version    downloads",
                     "-----  ---------  -----------",""]))
    assert out == res


def test_print_packages(package_list, capsys):
    print_packages(package_list)
    out, err = capsys.readouterr()
    res = unicode("\n".join(["app    version      downloads",
                             "-----  ---------  -----------",
                             "o1/p1  1.4.0               45",
                             "o1/p2  1.4.0               45", ""]))

    assert out == res


def test_print_empty_deploy_result(capsys):
    print_deploy_result([])
    out, err = capsys.readouterr()
    res = u'\n'.join(["\n", "package    version    type    name    namespace    status",
                      "---------  ---------  ------  ------  -----------  --------", ""])
    assert out == res


def test_print_deploy_result(deploy_result, capsys):
    print_deploy_result(deploy_result)
    out, err = capsys.readouterr()
    res = "\n".join(["\n",
                     "package    version    type                   name    namespace",
                     "---------  ---------  ---------------------  ------  -----------",
                     "p1         1.4.0      replicationcontroller  testns  \x1b[32mok\x1b[0m",
                     "p1         1.4.0      svc                    testns  \x1b[33mupdated\x1b[0m",
                     ""])
    assert out == res



import os.path
from kpm.utils import mkdir_p


class KpmAuth(object):
    def __init__(self):
        path = ".kpm/auth_token"
        home = os.path.expanduser("~")
        mkdir_p(os.path.join(home, ".kpm"))
        self.tokenfile = os.path.join(home, path)
        self._token = None

    @property
    def token(self):
        if self._token is None:
            if os.path.exists(self.tokenfile):
                f = open(self.tokenfile, 'r')
                self._token = f.read()
                f.close()
            else:
                return None
        return self._token

    @token.setter
    def token(self, value):
        self._token = value
        f = open(self.tokenfile, 'w')
        f.write(value)
        f.close()

    def delete_token(self):
        prev_token = self.token
        if os.path.exists(self.tokenfile):
            os.remove(self.tokenfile)
        self._token = None
        return prev_token
